/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010-2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.links.LinkDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo._base.event");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.Select");

dojo.require("jazz.ui.MessageBox");

dojo.require("com.ibm.rdm.web.common.Dialog");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.StatusDisplay");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.links.LinkArtifactPane");
dojo.require("com.ibm.rdm.web.links.Link");
dojo.require("com.ibm.rdm.web.links.LinkType");
dojo.require("com.ibm.rdm.web.links.LinkTypeUtils");
dojo.require("com.ibm.rdm.web.links.LinkTypeRole");
dojo.require("com.ibm.rdm.web.server.client.utils.LinkUtils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.calm.CalmRegistry");
dojo.require("com.ibm.rdm.web.calm.CalmLinkUtils");
dojo.require("com.ibm.rdm.web.calm.CalmConstants");
dojo.require("com.ibm.rdm.web.calm.CalmDialogWidget");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.server.cache.ProcessOperationsCache");
dojo.require("com.ibm.rdm.web.util.bidiutils"); //bidi
dojo.require("com.ibm.rdm.web.calm.CalmRegistry");
dojo.require("com.ibm.rdm.web.common.ArtifactIcons");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");

dojo.requireLocalization("com.ibm.rdm.web.links", "LinkDialog");


var ProcessOperationsCache = com.ibm.rdm.web.server.cache.ProcessOperationsCache;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var StatusDisplay = com.ibm.rdm.web.common.StatusDisplay;
var LinkTypeUtils = com.ibm.rdm.web.links.LinkTypeUtils;
var LinkUtils = com.ibm.rdm.web.server.client.utils.LinkUtils;
var LinkTypeRole = com.ibm.rdm.web.links.LinkTypeRole;
var CalmRegistry = com.ibm.rdm.web.calm.CalmRegistry;
var CalmLinkUtils = com.ibm.rdm.web.calm.CalmLinkUtils;
var domutils = com.ibm.rdm.web.util.domutils;
var CalmConstants = com.ibm.rdm.web.calm.CalmConstants;
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;
var MINIMUM_VIEWPORT_HEIGHT = 600;
// IMPORTANT NOTE: Any changes to the default dialog height and defatult dialog width values also need to be reflected in height and
// width hints of the CLM pickers.  See changesets for work item 67530.
var DEFAULT_DIALOG_HEIGHT   = 600;
var DEFAULT_DIALOG_WIDTH    = 824;
var DEFAULT_MESSAGE_BOX_HEIGHT = 30;
var BidiUtils = com.ibm.rdm.web.util.bidiutils;  //bidi
var SELECT_TYPE_VALUE = "#select";
var MORE_TYPE_VALUE = "#more";
var NONE_TYPE_VALUE = "#none";
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;


dojo.declare("com.ibm.rdm.web.links.LinkDialog", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "links/templates/LinkDialog.html"),
	
	// Dojo Attach Points
	contentNode: null,
	
	// Public fields 
	// default values should be mimicked in PersistentLinkDialog.js
	//If both are provided, resourceURI is used.
	resourceURI: null, //The resource to link to OR
	resourceURIs: null, //An array of resource to link to
	
	resourceInfo: null,
	resourceInfos: null,
	
	resourceName: null,
	projectURI: null,
	processAreaURI: null,
	artifactTypeURI: null,
	// Used by Diagram Editor and LinksExplorer when adding a link, code assumes this also implies
	// that a link must be selected
	forcePreferredLinkTypes: null,
	linkTypeURI: null,
	// used to identify the direction of the link when a linkTypeURI is specified
	resourceIsObject: false,
	filterArtifacts: [],
	callback: null,
	isCollection: false,
	allowMultipleSelection: true,
	isRichTextLink: false,
	allowNoLinkType: false,
	hyperLinkText: "",
	request: null,
	includeCLM: true,
	linkTypeWhitelist: null,
	modal: true,	
	// temporary variable for use as link constraints are adopted across the UI
	isApplyLinkConstraints: false,
	
	// Private fields
	_messages: null,
	_dialog: null,
	_dialogWidth: DEFAULT_DIALOG_WIDTH,
	_dialogHeight: DEFAULT_DIALOG_HEIGHT, 
	_messageBox: null,
	_pane: null,
	_linkTypes: null,
	// used for link contraints, the roles contain valid target artifact types for each role
	_linkTypeRoleTuples: null,
	// used for link constraints, a map by project id of the cross-project roles that are available and the valid target artifact types for each role
	_crossProjectLinkTypeRoles: null,
	_okButton: null,
	_cancelButton: null,
	_linkTypeSelect: null,
	_createdLinks: [],
	_errorLinks: [],
	_calmLinkTypes: [],
	_selectedItem: null,
	_isLinkingToArtifactValid: false,  	// cache whether or not linking to artifact values valid, used for ok enablement when link type changes
	_isLinkingToURLValid: false,		// cache whether or not link to web values valid, used for ok enablement when link type changes
	_linkTypeRequiredNode: null,
	_createArtifactNode: null,
	
	_currentlySelectedLinkType: null,
	_targetProject: null,
	
	_linkArtifactPane: null,
	
	_showAllLinkTypes: false,			// used to determine when to show all available link types and not just the preferred ones. will be set to
										// true when the More... option is selected and also when linkTypeURI has not been specified, this means that
										// the More... option has been selected from the Links sidebar/grid menu so we should show all available link types

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.links", "LinkDialog");		
	},
	
	postCreate: function() {
		this.inherited(arguments);

		this._messageBox = new jazz.ui.MessageBox(null, this._messageBoxDiv);
		this._currentlySelectedLinkType = this.linkTypeURI;
		this._targetProject = this.projectURI;
		this._crossProjectLinkTypeRoles = {};
		
		this._setupResourceInfo();
				
		this._linkTypeSelect = new dijit.form.Select({
			onChange: dojo.hitch(this, this._linkTypeChanged)
		});
		this._linkTypeNode.appendChild(this._linkTypeSelect.domNode);

		this._setupRichTextLink();
		
		// Add the Create New Artifact... link
		var aTag = dojo.create("a", {
			onclick: dojo.hitch(this, function(evt) {
				dojo.stopEvent(evt);
				this._handleCreateNewArtifactClicked();
			}), 
			href: "#", 
			tabIndex: 0
		});
		aTag.appendChild(document.createTextNode(this._messages.createNewArtifact));
		this._createArtifactNode.appendChild(aTag);
		this._createArtifactNode.style.display = "none";
		
		this._okButton = dojo.create("button", {
			innerHTML:this._messages.okButton,
			"class" : "button-primary",
			disabled: true
		});
		this.connect(this._okButton, "click", dojo.hitch(this, this.okClicked));
		this._buttonNode.appendChild(this._okButton);

		this._cancelButton = dojo.create("button", {
			innerHTML:this._messages.cancelButton
		});
		this.connect(this._cancelButton, "click", dojo.hitch(this, this.cancelClicked));
		this._buttonNode.appendChild(this._cancelButton);
	},
	
	_setupResourceInfo: function(){
		
		this._artifactNameNode.innerHTML = "";
		
		if(this.resourceInfos && (this.resourceInfos.length > 0)) {
			if (this.resourceInfos.length > 1) {
				dojo.addClass(this._artifactNameNode, "linking-from-multiple");
				var multipleMessage = dojo.string.substitute(this._messages.angleBracketsMessage, [this._messages.linkingFromMultipleArtifacts]);
				this._artifactNameNode.appendChild(document.createTextNode(multipleMessage));
				BidiUtils.applyTextDirSetting(this._artifactNameNode, multipleMessage, true);
				// Show the selected artifacts in the title attribute.
				var selectedArtifacts = "";
				dojo.forEach(this.resourceInfos, function(resourceInfo){
					if (!resourceInfo.name) {
						resourceInfo.name = resourceInfo.getDisplayTitle();
					}
					if (!resourceInfo.identifier) {
						resourceInfo.identifier = resourceInfo.getID();
					}
					var nameText = dojo.string.substitute(this._messages.artifactLinkingFrom, [resourceInfo.identifier, resourceInfo.name]);
					if (selectedArtifacts !== "") {
						selectedArtifacts = selectedArtifacts + ", " + nameText;
					} else {
						selectedArtifacts = nameText;
					}
				}, this);	
				this._artifactNameNode.setAttribute("title", selectedArtifacts);
			} else {
				var resourceInfo = this.resourceInfos[0];
				if (!resourceInfo.name) {
					resourceInfo.name = resourceInfo.getDisplayTitle();
				}
				if (!resourceInfo.identifier) {
					resourceInfo.identifier = resourceInfo.getID();
				}
				if (!resourceInfo.ofType) {
					resourceInfo.ofType = resourceInfo.getType();
				}
				var artifactIcon = com.ibm.rdm.web.common.ArtifactIcons.getSmallImage(this.projectURI, resourceInfo.ofType);
				dojo.addClass(artifactIcon, "linking-from-icon");
				this._artifactNameNode.appendChild(artifactIcon);
				var separ = BidiUtils.getSeparator();
				var nameText = dojo.string.substitute(this._messages.artifactLinkingFrom, [resourceInfo.identifier, separ + BidiUtils.setTextDirUCC(resourceInfo.name)]);
				this._artifactNameNode.appendChild(document.createTextNode(nameText));				
			} 
		}
		// fall back
		else {			
			this._artifactNameNode.appendChild(document.createTextNode(this.resourceName));
			//bidi
			BidiUtils.applyTextDirSetting(this._artifactNameNode, this.resourceName, true);
		}
	},
	
	_setupRichTextLink: function(){
		if (this.isRichTextLink) {
			this._hyperLinkTextField.value = this.hyperLinkText;
			// set link label field...
		} else {
			this._hyperLinkTextRow.style.display = "none";
			dojo.addClass(this._hyperLinkTable, "hyperlink-table-hide");
		}
	},
	
	_handleCreateNewArtifactClicked: function() {
		// summary:
		//		Handle the Create New Artifact action.
		
		this._linkArtifactPane.createNewArtifact();
	},
	
	onOpen: function() {
		// summary:
		//		Callback to handle the case of the dialog being opened.
		// description:
		//		Overload this function if you'd like to be called back when
		//		the dialog is opened.
		// tags:
		//		callback
	},
	
	onClose: function() {
		// summary:
		//		Callback to handle the case of the "Close" button being
		//		pressed.
		// description:
		//		Overload this function if you'd like to be called back when
		//		the dialog is closed as a result of the "Close" button being
		//		pressed.
		// tags:
		//		callback
	},
	
	onCancel: function() {
		// summary:
		//		Callback to handle the case of the "Cancel" button being
		//		pressed.
		// description:
		//		Overload this function if you'd like to be called back when
		//		the dialog is closed as a result of the "Cancel" button being
		//		pressed.
		// tags:
		//		callback
	},
	
	uninitialize: function(){
		this._messageBox.destroyRecursive();
		if (this._linkArtifactPane) {
			this._linkArtifactPane.destroyRecursive();
		}
		this._linkTypeSelect.destroyRecursive();	
	},
	
	show: function(/*Callback?*/onDialogClose) {
		var self= this;
		var projectOperationsDeferred = ProjectCache.getProjectPermissions(this.projectURI);
		projectOperationsDeferred.addCallback(this, function(projectOperations){
			var canCreateLinks = projectOperations.item(ProjectCache.MODIFY_RESOURCE);
			// Create the Dialog	
			
			if (!canCreateLinks) {
				this._dialogHeight += 35;
			} 
			if (this.isRichTextLink) {
				// rich text linking adds an additional field to the dialog
				this._dialogHeight += 20;
			}		
			this._adjustDialogHeight();
			
			// TODO Remove after Foundation has a fix for work item 34954
			var content = document.createElement("div");
			content.id = "content_" + this.contentNode.id;
			var backgroundFrame = new dijit.BackgroundIframe(content);
			content.appendChild(this.contentNode);
			
			this._dialog = new com.ibm.rdm.web.common.Dialog({
				contentNode: this.domNode,
				destroyContents: true,
				primaryTitle: this._messages.createLinkDialogTitle,
				width: this._dialogWidth + "px",
				height: this._dialogHeight + "px",
				onOpen: this.onOpen,
				onClose: this.onClose,
				onDialogClose: onDialogClose,
				modal: this.modal
			});
			
			dojo.style(this._dialog, "overflow", "auto");
			this._initLinkTypes();

			return projectOperations;
		});
	},
	
	hideMessage: function() {
		this._messageBox.hideMessage();
		this._resetDialogSize();
	},
	
	message: function(messageId, messageParms) {
		this._resetDialogSize(null, this._dialogHeight + DEFAULT_MESSAGE_BOX_HEIGHT);
		this._messageBox.showMessage(com.ibm.rdm.web.common.MessageHandler.getMessageBoxInfo(messageId, messageParms));
	},
	
	_linkTypeChanged: function() {
		this._currentlySelectedLinkType = this._linkTypeSelect.get("value");
		if (this._currentlySelectedLinkType === null) {
			this.message(7666);
			this._linkTypeSelect.set("disabled", true);
			this._dialogHeight = 150;
			this._resetDialogSize();
			return;
		} else if (this._currentlySelectedLinkType === MORE_TYPE_VALUE) {
			this.forcePreferredLinkTypes = false;
			// the More... option is selected, _showAllLinkTypes is set to true
			this._showAllLinkTypes = true;
			// clear the Link Types select options
			this._linkTypeSelect.removeOption(this._linkTypeSelect.getOptions());
			this._initLinkTypes();
			this._linkTypeSelect.loadAndOpenDropDown();
			return;
		}
		this.hideMessage();
		var pane = this._getPaneForLinkType();
		if (pane.showPickerDialogs) {
			this._pane = pane;
			this.hideButtonArea();
			domutils.removeChildNodes(this._contentAreaNode);
			this._contentAreaNode.appendChild(this._pane.domNode);
			this._pane.showPickerDialogs();
		} else {
			this._resetDialogSize();
			this.showButtonArea();
			if (pane !== this._pane) {
				this.okEnablement(false);
				this._pane = pane;
				domutils.removeChildNodes(this._contentAreaNode);
				this._contentAreaNode.appendChild(this._pane.domNode);
				pane.startup();
			} else {
				this._checkOkEnablementOnLinkTypeChanged();
			}
			if (this.isApplyLinkConstraints) {
				this._getTargetArtifactTypesForSelectedLinkRole().then(dojo.hitch(this, function(types) {
					LinkTypeUtils.getLinkConstraintsCache().areConstraintsDefinedForProject(this._targetProject).then(dojo.hitch(this, function(defined) {
						if (defined) {
							this._linkArtifactPane.setTargetArtifactTypes(types);
						}
						// enable web linking based on target artifact types
						this._enableLinkToWeb(this._isLinkToWebAllowedForSelectedLinkRole());
						// enable cross project linking based on target artifact types
						var selectedLinkType = this._getLinkTypeForSelection();
						var enable = selectedLinkType && !selectedLinkType.isCustomDefined();
						if (enable && this._isXProjectAllowedForSelectedLinkRole()) {
							this._setProjectPickerToUseAllProjects();
						} else {
							this._setProjectPickerToBeRestrictedToProject(this.projectURI);
						}
					}));
				}));
			} else {
				var selectedLinkType = this._getLinkTypeForSelection();
				if (selectedLinkType && selectedLinkType.isCustomDefined())  {
					// It's a custom link type.
					this._setProjectPickerToBeRestrictedToProject(this.projectURI);
				} else {
					// It's not a custom link type.
					this._setProjectPickerToUseAllProjects();
				}
			}
		}
		
		// If we have one selected item, must check link permissions when the link type
		// is changed.  Be sure not to do this when linking to a url.  Currently, there
		// is no API to get selected item (as an object) from the LinkArtifactPane, so use
		// a cached value.
		if (this._selectedItem && !this._linkArtifactPane.isLinkingToURL()) {
			this.checkLinkPermission(this._selectedItem);
		}
	},
	
	_isLinkToWebAllowedForSelectedLinkRole: function() {
		// Return whether or not linking to the Web is allowed for the selected link
		// type based on link constraints.
		
		if (this._currentlySelectedLinkType === SELECT_TYPE_VALUE) {
			return false;
		}
		
		// if target artifacts are not defined, the link type is not constrained
		var targets = this._getTargetArtifactTypesForLinkSelection();
		if (!targets) {
			return true;
		}
		return dojo.indexOf(targets, RRMTypes.WEB_ARTIFACT_TYPE_URI) !== -1;
	},
	
	_isXProjectAllowedForSelectedLinkRole: function() {
		// Return whether or not cross-project linking is allowed for the selected link
		// type based on link constraints.
		
		if (this._currentlySelectedLinkType === SELECT_TYPE_VALUE) {
			return false;
		}
		
		// if target artifacts are not defined, the link type is not constrained
		var targets = this._getTargetArtifactTypesForLinkSelection();
		if (!targets) {
			return true;
		}
		return dojo.indexOf(targets, RRMTypes.XPROJECT_ARTIFACT_TYPE_URI) !== -1;
	},
	
	_getTargetArtifactTypesForLinkSelection: function() {
		// Return the list of target artifact type URIs that are valid for the selected link role.
		
		var linkType = this._getLinkTypeForSelection();
		var isObject = this._currentlySelectedLinkType.indexOf("/object") !== -1;
		for (var i=0; i<this._linkTypeRoleTuples.length; i++) {
			var tuple = this._linkTypeRoleTuples[i];
			for (var j=0; j<tuple.length; j++) {
				var role = tuple[j];
				if (role.getLinkType().getResourceURI() === linkType.getResourceURI()) {
					if ((role.isObjectRole() && isObject) || (role.isSubjectRole() && !isObject))  {
						if (role.isTargetConstrained()) {
							return role.getValidTargetArtifactTypeURIs();
						} else {
							return null;
						}
					}
				}
			}
		}
		return null;
	},
	
	_getAvailableCrossProjectCreationLinkTypeRoles: function(crossProjectURI) {
		// summary
		// 		Return, via a deferred, the linkTypeRole tuples that are available for cross-project linking for the given
		//		project.
		
		var deferred = new dojo.Deferred();
		var tuples = this._crossProjectLinkTypeRoles[crossProjectURI];
		if (tuples) {
			deferred.callback(tuples);
		} else {
			LinkTypeUtils.getAvailableConstrainedCrossProjectCreationLinkTypesRoles(crossProjectURI, crossProjectURI).then(dojo.hitch(this, function(availableRoles){
				this._crossProjectLinkTypeRoles[crossProjectURI] = availableRoles;
				deferred.callback(availableRoles);
			}));
		}
		return deferred;
	},
	
	_getCrossProjectTargetArtifactTypesForLinkSelection: function(crossProjectURI) {
		// summary
		// 		Return, via a deferred, the target artifact types that are available for cross-project linking for the given
		//		project and the current link type selection.
		
		var deferred = new dojo.Deferred();
		if (this._currentlySelectedLinkType === SELECT_TYPE_VALUE) {
			deferred.callback(null);
		} else {
			var targets = [];
			var linkType = this._getLinkTypeForSelection();
			var isSubject = this._currentlySelectedLinkType.indexOf("/subject") !== -1;
			var selectedLinkTypeRole = new LinkTypeRole({linkType: linkType, isSubjectRole: isSubject});
			this._getAvailableCrossProjectCreationLinkTypeRoles(crossProjectURI).then(dojo.hitch(this, function(roles) {
				var found = false;
				for (var i=0; i<roles.length; i++) {
					var role = roles[i];
					if (selectedLinkTypeRole.isAvailableForCrossProjectLinkingTo(role)) {
						found = true;
						targets = role.getValidTargetArtifactTypeURIs();
						break;
					}
				}
				// if the selected role is not found in the cross-project tuples, no linking is allowed due to link constraints
				var linkTypeLabel = "";
				if (selectedLinkTypeRole.isSubjectRole()) {
					linkTypeLabel = linkType.getSubjectToObjectLabel();
				} else {
					linkTypeLabel = linkType.getObjectToSubjectLabel();
				}
				if (this._isXProjectAllowedForSelectedLinkRole()) {
					var projectName = this._linkArtifactPane.getSelectedProjectName();
					if (!found) {
						this.message(7910, [linkTypeLabel, projectName]);
					}
				}
				deferred.callback(targets);
			}));
		}
		return deferred;
	},
	
	_getTargetArtifactTypesForSelectedLinkRole: function() {
		// Return, via a deferred, the list of target artifact type URIs that are valid for the selected link role
		// and project.
		
		if (!this.isApplyLinkConstraints) {
			return;
		}
		var deferred = new dojo.Deferred();
		if (this._currentlySelectedLinkType === SELECT_TYPE_VALUE) {
			deferred.callback(null);
		} else {
			var linkType = this._getLinkTypeForSelection();
			if (this.projectURI !== this._targetProject && linkType.isSystemDefined()) {
				this._getCrossProjectTargetArtifactTypesForLinkSelection(this._targetProject).then(dojo.hitch(this, function(targets) {
					deferred.callback(targets);
				}));			
			} else {
				deferred.callback(this._getTargetArtifactTypesForLinkSelection());
			}
		}
		return deferred;
	},
	
	_getSingleOption: function() {
		// If there is only one link type role available for selection, return it.
		if (this._linkTypeRoleTuples.length === 1 && this._linkTypeRoleTuples[0].length === 1) {
			return (this._linkTypeRoleTuples[0])[0];
		}
		return null;
	},
	
	_getPaneForLinkType: function(reload) {
		var calmLinkType = this._calmLinkTypes[this._currentlySelectedLinkType];
		if (calmLinkType) {
			if (!this.resourceURI) {
				this.resourceURI = this.resourceURIs[0];
			}
			var calmDialogWidget = new com.ibm.rdm.web.calm.CalmDialogWidget({
				"resourceURI": this.resourceURI,
				"resourceInfos": this.resourceInfos,
				"projectURI": this.projectURI,
				"viewer": this,
				"isCollection": this.isCollection,
				"isRichTextLink": this.isRichTextLink
			});
			calmDialogWidget.registerConsumer(this);
			calmDialogWidget.setSelectedLinkType(this._currentlySelectedLinkType);
			return calmDialogWidget;
		}
		if (this._linkArtifactPane === null || reload) {
			var targetTypes = null;
			var onProjectChange = null;
			if (this.isApplyLinkConstraints) {
				onProjectChange = dojo.hitch(this, this._projectChanged);
			}
			this._linkArtifactPane = new com.ibm.rdm.web.links.LinkArtifactPane({
				linkDialog: this, 
				allowMultipleSelection: this.allowMultipleSelection,
				projectURI: this.projectURI,
				linkLabelText: this.hyperLinkText,	// seed Link Label field with hyperLinkText if specified
				createArtifactNode: this._createArtifactNode,
				onProjectChange: onProjectChange
			});
		}
		return this._linkArtifactPane;
	},
	
	_projectChanged: function(projectURI) {
		// The selected project has changed.  Update the target artifact types that are valid for 
		// the selected link.
		
		if (!this.isApplyLinkConstraints) {
			return;
		}
		this.hideMessage();
		this._targetProject = projectURI;
		this._getTargetArtifactTypesForSelectedLinkRole().then(dojo.hitch(this, function(targetTypes) {
			this._linkArtifactPane.setTargetArtifactTypes(targetTypes);
		}));
	},
	
	itemSelected: function(item) {
		this.hideMessage();
		// cache the single selected item for using when linkType changes, currently there
		// is no API to get the selected item (as an object) from LinkArtifactPane
		this._selectedItem = item;
		this.checkLinkPermission(item);
	},
	
	_getLinkTypeForSelection: function() {
		var index = this._currentlySelectedLinkType.lastIndexOf("/");
		if (index === -1) {
			return null;
		}
		var selectedURI = this._currentlySelectedLinkType.substring(0, index);
		return this._linkTypes[selectedURI];
	},
	
	_getLinkTypeFromURI: function(linkTypeURI){
		return this._linkTypes[linkTypeURI];
	},
	
	checkLinkPermission: function(item) {
		// Check link permission, ignore CLM links as permissions for these are handled differently.
		// Permission checking is only done when one link is selected.
		var calmLinkType = this._calmLinkTypes[this._currentlySelectedLinkType];
		if (calmLinkType) {
			return;
		}
		if (!this.processAreaURI) {
			return;
		}
		var cmEnabled = ProjectCache.getCmState(this.projectURI);
	     if ( cmEnabled && ( item.projectURI !== this.projectURI ) ) {
	    	 return; //Do not check for Opt-in projects
	     }
		var linkType = this._getLinkTypeForSelection();
		if (!linkType) {
			return;
		}
		if (item.projectURI !== this.projectURI) {
			if (!linkType.isSystemDefined()) {
				// permissions can only be enforced when cross-project linking if the link type is system defined.
				return;
			}
		}
		var permissionsDeferred = ProcessOperationsCache.isCreateLinkAllowed(item.projectURI, linkType.getActionId(), item.processAreaURI);
		permissionsDeferred.addCallback(this, function(allowed) {
			if (!allowed) {
				dojo.attr(this._okButton, "disabled", true);
				this.message(7662, [linkType.getName(), item.label]);
			} else  {
				dojo.attr(this._okButton, "disabled", false);
			}			
		});
	},
	
	artifactsSelected: function(artifacts) {
		if (artifacts.length > 0) {
			this.hideMessage();
		}
		this._selectedItem = null;
		this.okEnablement(artifacts.length > 0);
	},
	
	isOkEnabled: function() {
		return ! dojo.attr(this._okButton, "disabled");			
	},
	
	_checkOkEnablementOnLinkTypeChanged: function() {
		// Checks the state of the dialog when the link type changes to see if the OK button should be enabled.
		// The dialog was written with the assumption that a valid link type value would always exist, so this
		// method is used to work around that assumption.
		
		if (this._currentlySelectedLinkType && (this._currentlySelectedLinkType === SELECT_TYPE_VALUE)) {
			dojo.attr(this._okButton, "disabled", true);
			return;
		} 
		if (this._linkArtifactPane) {
			if (this._linkArtifactPane.isLinkingToURL()) {
				if (!this._isLinkingToURLValid) {
					dojo.attr(this._okButton, "disabled", true);
					return;
				}
			} else {
				if (!this._isLinkingToArtifactValid) {
					dojo.attr(this._okButton, "disabled", true);
					return;
				}
			}
			// Unclear why the dialog would even be opened if a link cannot be created, but put this
			// test in since okEnablement has the test.
			var projectOperationsDeferred = ProjectCache.getProjectPermissions(this.projectURI);
			projectOperationsDeferred.addCallback(this, function(projectOperations){
				var canCreateLinks = projectOperations.item(ProjectCache.MODIFY_RESOURCE);
				dojo.attr(this._okButton, "disabled", !canCreateLinks);
				return projectOperations;
			});
		}
	},
	
	linkingToToggled: function(isLinkingToArtifact) {
		// summary:
		//		The type of linking that is occurring has been changed.
		//		When linking to the web, the Create New Artifact option 
		//		should not be available.
		this.hideMessage();
		if (isLinkingToArtifact) {
			this._createArtifactNode.style.display = "";
		} else {
			this._createArtifactNode.style.display = "none";
		}
	},

	okEnablement: function(enabled) {
		// Cache the valid state of the particular link pane for use when the link type changes.
		if (this._linkArtifactPane.isLinkingToURL()) {
			this._isLinkingToURLValid = enabled;
		} else {
			this._isLinkingToArtifactValid = enabled;
		}
		if (this._currentlySelectedLinkType && (this._currentlySelectedLinkType === SELECT_TYPE_VALUE)) {
			dojo.attr(this._okButton, "disabled", true);
			return;
		}
		var projectOperationsDeferred = ProjectCache.getProjectPermissions(this.projectURI);
		projectOperationsDeferred.addCallback(this, function(projectOperations){
			var canCreateLinks = projectOperations.item(ProjectCache.MODIFY_RESOURCE);
			dojo.attr(this._okButton, "disabled", !canCreateLinks || !enabled);
			return projectOperations;
		});
	},
	
	okClicked: function() {
		// disable the ok button so the user cannot click it again
		dojo.attr(this._okButton, "disabled", true);
		// also disable the cancel button so it cannot be clicked after ok
		dojo.attr(this._cancelButton, "disabled", true);
		var linkText = null;
		var linkLabel = null;
		if (this.isRichTextLink) {
			linkText = this._hyperLinkTextField.value;
			if (this._pane.isLinkingToURL()) {
				var label = this._pane.getLinkLabel();
				if (label !== "") {
					linkLabel = label;
				}
			}
		}
		this.linksSelected(this._pane.getSelectedLinks(), this._pane.getSelectedLinkNames(), linkText, linkLabel);
	},
	
	linksSelected: function(uris, names, hyperLinkText, linkLabel) {
		// Determine whether the resource URI should be subject or object
		var linkType = this._getLinkTypeForSelection();
		var isObject = this._currentlySelectedLinkType.indexOf("/object") !== -1;
		var linkValues = this._pane.getSelectedLinkValues();

		// Create the links
		this._createdLinks = [];	
		this._errorLinks = [];
		var toCreateLinks = [];	
		var linksToCreate = [];
		StatusDisplay.showStatusMessage(this._messages.savingLinks);
		this._resetDialogSize(null, this._dialogHeight + DEFAULT_MESSAGE_BOX_HEIGHT);
		this._messageBox.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: this._messages.savingLinks,
			details: [],
			closable: true
		});
		
		//If we only have one resource, make it an array of one.
		if (this.resourceURI) {
			this.resourceURIs = [this.resourceURI];
		}
		
		if (linkType === null && !this.isRichTextLink) {
			// no traceability link selected
		} else {
			for (var i = 0; i < this.resourceURIs.length; i++) {
				for (var j = 0; j < uris.length; j++) {
					var subjectURI = isObject ? uris[j] : this.resourceURIs[i];
					var objectURI = isObject ? this.resourceURIs[i] : uris[j];
					var link = new com.ibm.rdm.web.links.Link({
						subject: subjectURI,
						object: objectURI,
						title: linkValues[j],
						linkType: linkType ? linkType.getResourceURI() : null,
						projectURI: linkType ? linkType.getProjectURI() : null
					});
					if (this.isRichTextLink) {
						// if a hyperlink is being created, the rich text editor will handle creating
						// the link object
						toCreateLinks.push(link);
					} else {
						if (this.request !== null && this.request.requestData !== null) {
							if (this.request.requestData.locationId !== null) {
								if (isObject) {
									link._objectLocationId = this.request.requestData.locationId;
								} else {
									link._subjectLocationId = this.request.requestData.locationId;
								}
							}
							if (this.request.requestData.description !== null) {
								if (isObject) {
									link._objectDescription = this.request.requestData.description;
								} else {
									link._subjectDescription = this.request.requestData.description;
								}
							}
						}
						linksToCreate.push(link);
					}
				}
			}
		}
		if (hyperLinkText) {
			hyperLinkText = domutils.getXMLEncodedText(hyperLinkText);
		}
		if (this.isRichTextLink) {
			this.callback(toCreateLinks, names, hyperLinkText, linkLabel);
			StatusDisplay.hideStatusMessage();
		} else {
			if (linksToCreate.length === 0) {
				// handle callback for the "No Traceability Link" case
				if (this.callback) {
					// links to create will be empty, names will be an array of artifact names for the
					// selected uris, hyperLinkText and linkLabel will be as specified in dialog,
					// uris will be array of selected artifact uris
					this.callback(linksToCreate, names, hyperLinkText, linkLabel, uris);
					this.close();
					StatusDisplay.hideStatusMessage();
				}
			} else {
        		this.domNode.style.cursor = 'wait';
				var createLinksDeferred = this._createLinks(linksToCreate);
				createLinksDeferred.addCallback(dojo.hitch(this, function() {
					if (this.callback) {
						this.callback(this._createdLinks, names, hyperLinkText);
					}
					this.domNode.style.cursor = 'auto';
                	// If errors occurred during creation of multiple links, leave the dialog open and show the
                	// user the error message.  
					if (this._errorLinks.length === 0) {
						this.close();
					} else {
						com.ibm.rdm.web.common.MessageHandler.showXhrError(this._errorLinks[0], this._errorLinks[0].ioArgs, this._messageBox);
						// Something went wrong, make sure cancel is enabled
						dojo.attr(this._cancelButton, 'disabled', false);
					}
					StatusDisplay.hideStatusMessage();
				}));
				createLinksDeferred.addErrback(dojo.hitch(this, function(response) {
                	this.domNode.style.cursor = 'auto';
					com.ibm.rdm.web.common.MessageHandler.showXhrError(response, response.ioArgs, this._messageBox);
					StatusDisplay.hideStatusMessage();
					// Something went wrong, make sure cancel is enabled
					dojo.attr(this._cancelButton, 'disabled', false);
				}));
			}
		}
	},
	
	_createLinks: function(links) {
		var self = this;
		var deferred= new dojo.Deferred();
		var createDeferred = null;
		//If we are creating more than one link, create the links in a batch by calling LinkUtils.saveLinks()
		if (links.length > 1) { 
			// skipErrors when saving the links so that we can display server error messages in
			// the dialog and so that we can handle partially successful link creation
			createDeferred = LinkUtils.saveLinks(this.projectURI, links, false, false, true);
			createDeferred.addCallback(function(linksResponse){
				for (var key in linksResponse) {
					if (linksResponse[key][0]) {
						self._createdLinks.push(linksResponse[key][1]);	
					} else {
						self._errorLinks.push(linksResponse[key][1]);	
					}
				}
				deferred.callback();
			});			
			createDeferred.addErrback(function(response){
				deferred.errback(response);
			});			
		}
		else {
			// skipErrors when saving the link so that we can display server error messages in
			// the dialog
			createDeferred = LinkUtils.saveLink(this.projectURI, links[0], false, true);
			createDeferred.addCallback(function(link){
				self._createdLinks.push(link);
				deferred.callback();
			});			
			createDeferred.addErrback(function(response){
				deferred.errback(response);
			});			
		}		
		return deferred;
	},
	
	close: function() {
		this._dialog.close();
	},
	
	cancelClicked: function() {
		this.onCancel();
		this.close();
	},
	
	_doGetProjectAreaLinks: function(resourceProjectURI, callbackDeferred){
		//Get the cross repository links for the current project
		var prjCalmLinks = CalmRegistry.getProjectAreaLinks(resourceProjectURI);
		
		prjCalmLinks.addCallback(this, function(links){	
			//return ProjectAreaLink types
			callbackDeferred.callback(links);	
		});
		prjCalmLinks.addErrback(this,function(errorResponse){
			callbackDeferred.errback(errorResponse);
		});
	},
	
	hideButtonArea: function() {
		// summary:
		//		Hide the Ok/Cancel buttons and Create New Artifact link. 
		//		The button area is hidden for the CLM instances of the link dialog.
		
		this._buttonNode.style.display = "none";
		this._createArtifactNode.style.display = "none";
	},
	
	showButtonArea: function() {
		// summary:
		//		Show the Ok/Cancel buttons and Create New Artifact link. 
		//		The button area is only show for non-CLM instances of the link dialog.
		
		this._buttonNode.style.display = "";
		if (!this._linkArtifactPane.isLinkingToURL()) {
			this._createArtifactNode.style.display = "";
		} else {
			this._createArtifactNode.style.display = "none";
		}
	},
	
	_isAlreadyInOptionsList: function(vector, newMember /* {label, value} */) {
		var isAlreadyInOptionsList = false;
		for (var index in vector) {
			if ((vector[index].label == newMember.label)
				&& (vector[index].value == newMember.value)
				) {
				isAlreadyInOptionsList = true;
				break;
			}
		}
		return isAlreadyInOptionsList;
	},
	
	_dialogWidthPadding : 50,
	_dialogHeightPadding : 150,
	_dialogWidthPaddingEm : 4,
	_dialogHeightPaddingEm : 13,
	_dialogSizeUnit: "px",
	_dialogSizeUnitEm: "em",
	signalNewPickerSize:function(width, height){
		var dlgWidth = 0;
		var dlgHeight = 0;
		var isEm = false;
		var pxIdx = width.indexOf(this._dialogSizeUnit);
		var widthPadding = this._dialogWidthPadding;
		var heightPadding = this._dialogHeightPadding;
		if (pxIdx === -1) {
			pxIdx = width.indexOf(this._dialogSizeUnitEm);
			if (pxIdx !== -1) {
				isEm = true;
				widthPadding = this._dialogWidthPaddingEm;
				heightPadding = this._dialogHeightPaddingEm;
			}
		}
		if (pxIdx > -1) {
			dlgWidth = parseInt(width.substring(0, pxIdx));
		}
		pxIdx = height.indexOf(this._dialogSizeUnit);
		if (pxIdx === -1) {
			pxIdx = height.indexOf(this._dialogSizeUnitEm);
		}
		if (pxIdx > -1) {
			dlgHeight = parseInt(height.substring(0, pxIdx));
		}
		dlgWidth += widthPadding;
		dlgHeight += heightPadding;

		var dialogUnit = isEm ? this._dialogSizeUnitEm : this._dialogSizeUnit;
		this._dialog.setWidth(dlgWidth + dialogUnit);
		this._dialog.setHeight(dlgHeight + dialogUnit);
		this.contentNode.style.width = width;
		if(this._contentAreaNode) {
			this._contentAreaNode.style.width = width;
			this._contentAreaNode.style.height = height;
		}
	},
	
	_resetDialogSize: function(widthToUse, heightToUse) {
		this._adjustDialogHeight();
		if (widthToUse) {
			this._dialog.setWidth(widthToUse + this._dialogSizeUnit);
		} else {
			this._dialog.setWidth(this._dialogWidth + this._dialogSizeUnit);
		}
		if (heightToUse) {
			this._dialog.setHeight(heightToUse + this._dialogSizeUnit);
		} else {
			this._dialog.setHeight(this._dialogHeight + this._dialogSizeUnit);
		}
	},
	
	_adjustDialogHeight: function(){
		//Dialog is to tall when the viewport is 1024x768 or smaller. Adjust the height to account for this:
		if(this._getViewPortDimensions().height < MINIMUM_VIEWPORT_HEIGHT){
			    //Reduce the dialog height by the difference between the current view port height and the minimum dialog height
				this._dialogHeight = DEFAULT_DIALOG_HEIGHT - ( MINIMUM_VIEWPORT_HEIGHT - this._getViewPortDimensions().height);
		}	
	},
	
	_getViewPortDimensions: function(){
		var viewportwidth;
		var viewportheight;
		
		//(mozilla/netscape/opera/IE7) use window.innerWidth and window.innerHeight
		
		if (typeof window.innerWidth !== 'undefined') {
			viewportwidth = window.innerWidth;
		    viewportheight = window.innerHeight;
		}
		
		// IE6 in standards compliant mode	
		else if (typeof document.documentElement !== 'undefined' &&
			typeof document.documentElement.clientWidth !== 'undefined' &&
			document.documentElement.clientWidth !== 0) {
				viewportwidth = document.documentElement.clientWidth;
				viewportheight = document.documentElement.clientHeight;
			}			
		// older versions of IE			
		else {
			viewportwidth = document.getElementsByTagName('body')[0].clientWidth;
			viewportheight = document.getElementsByTagName('body')[0].clientHeight;
		}
		
		return {height:viewportheight, width:viewportwidth};			
	},
	
	_initLinkTypes: function(skipLinkTypeChanged) {
		var self = this;
		this._crossProjectLinkTypeRoles = {};
		var linkTypesDeferred = LinkTypeUtils.getLinkTypesForProject(this.projectURI);
		var isPreferenceSpecified = false;
		var getOrCreateNoLinkType = function () {
			if (!com.ibm.rdm.web.links.LinkDialog.NO_LINK_TYPE) {
				com.ibm.rdm.web.links.LinkDialog.NO_LINK_TYPE = {
					label: domutils.getXMLEncodedText(self._messages.noLinkType),
					value: self._messages.noLinkType
				};
			}
			return com.ibm.rdm.web.links.LinkDialog.NO_LINK_TYPE;
		};
		
		var createSelectLinkType = function () {
			if (!com.ibm.rdm.web.links.LinkDialog.SELECT_LINK_TYPE) {
				com.ibm.rdm.web.links.LinkDialog.SELECT_LINK_TYPE = {
					label: domutils.getXMLEncodedText(self._messages.selectLinkType),
					value: SELECT_TYPE_VALUE
				};
			}
			return com.ibm.rdm.web.links.LinkDialog.SELECT_LINK_TYPE;
		};

		var linkTypeRolesDeferred = null;
		if (this.isApplyLinkConstraints) {
			var typeURIs = [];
			for (var i=0; i<this.resourceInfos.length; i++) {
				var resourceInfo = this.resourceInfos[i];
				if (resourceInfo.getType) {
					typeURIs.push(resourceInfo.getType());
				} else if (resourceInfo.ofType) {
					typeURIs.push(resourceInfo.ofType);
				}
			}
			linkTypeRolesDeferred = LinkTypeUtils.getAvailableConstrainedCreationLinkTypesRoles(this.projectURI, this.processAreaURI, typeURIs);
		} else {
			linkTypeRolesDeferred = LinkTypeUtils.getAvailableCreationLinkTypesRoles(this.projectURI, null, this.processAreaURI, this.artifactTypeURI);
		}
		linkTypeRolesDeferred.addCallback(this, function(linkTypeRoleTuples) {
			if (this.isApplyLinkConstraints) {
				this._linkTypeRoleTuples = linkTypeRoleTuples;
			}
			this._linkTypes = {};
			this._calmLinkTypes = {};
			var linkTypeOptions = [];
			var separatorOption = {
					type: 'separator'
				};
			if (this.isApplyLinkConstraints && linkTypeRoleTuples.length === 0) {
				linkTypeOptions.push({
					label: domutils.getXMLEncodedText(this._messages.noneAvailable),
					value: NONE_TYPE_VALUE
				});
				this._linkTypeSelect.addOption(linkTypeOptions);
				this._linkTypeSelect.set("disabled", true);
				if (!skipLinkTypeChanged) {
					this._linkTypeChanged();
				}
				return linkTypeRoleTuples;
			}
			var permissibleLinkTypes = false;
			for (var i=0; i<linkTypeRoleTuples.length; i++) {
				var tuple = linkTypeRoleTuples[i];
				for (var j=0; j<tuple.length; j++) {
					var role = tuple[j];
					if (role.isAllowed()) {
						permissibleLinkTypes = true;
						break;
					}
				}
			} 
			if (!permissibleLinkTypes) {
				linkTypeOptions.push({
					label: domutils.getXMLEncodedText(this._messages.noTypes),
					value: NONE_TYPE_VALUE
				});
				this._linkTypeSelect.addOption(linkTypeOptions);
				this._linkTypeSelect.set("disabled", true);
				if (!skipLinkTypeChanged) {
					this._linkTypeChanged();
				}
				return linkTypeRoleTuples;
			}
			if (this.allowNoLinkType) {
				linkTypeOptions.push(getOrCreateNoLinkType());
				linkTypeOptions.push(separatorOption);
			} else if (!this.linkTypeURI) {
				// if a linkTypeURI has not been specified, add an "empty" option, indicating that a link type must
				// be selected
				linkTypeOptions.push(createSelectLinkType());
				dojo.style(this._linkTypeRequiredNode, "display", "inline-block");
				linkTypeOptions.push(separatorOption);
				// the More... option is selected from Links sidebar/grid, _showAllLinkTypes is set to true
				this._showAllLinkTypes = true;
			}
			
			if(this._showAllLinkTypes && this.forcePreferredLinkTypes) {
				// override _showAllLinkTypes to false because we want to force preferred link types (i.e. LinksExplorer)
				this._showAllLinkTypes = false;
			}
			
			if (this.isApplyLinkConstraints && !this._currentlySelectedLinkType && this.forcePreferredLinkTypes) {
				// if only one options exists, select it in the case of forcePreferredLinkTypes 
				var singleLinkTypeRole = this._getSingleOption();
				if (singleLinkTypeRole) {
					// select this role
					this._currentlySelectedLinkType = singleLinkTypeRole.getLinkType().getResourceURI();
					this.resourceIsObject = singleLinkTypeRole.isObjectRole();
				}
			}

			var args = {
				linkTypeRoleTuples: linkTypeRoleTuples,
				linkTypeWhitelist: this.linkTypeWhitelist,
				separatorOption: separatorOption,
				selectedLinkType: this._currentlySelectedLinkType,
				showAllLinkTypes: this._showAllLinkTypes,
				includeCLM: this.includeCLM,
				fromMultiple: ((this.resourceURIs !== null) && (this.resourceURIs.length > 1)), 
				isObject: this.resourceIsObject, 
				isCollection: this.isCollection,
				isElementLevelLink: ((this.params.request && this.params.request.requestData.locationId && this.params.request.requestData.locationId !== ""))
			};
			
			var processedTuplesResults = LinkTypeUtils.processLinkTuples(args);
			
			linkTypeOptions = linkTypeOptions.concat(processedTuplesResults.linkTypeOptions);
			this._linkTypes = processedTuplesResults.linkTypes;
			this._calmLinkTypes = processedTuplesResults.calmLinkTypes;
			isPreferenceSpecified = processedTuplesResults.isPreferenceSpecified;
			
			if (linkTypeOptions.length === 0) {
				var noneOption = {
					label: this._messages.noTypes,
					value: null
				};
				linkTypeOptions.push(noneOption);
			} else if ((!this._showAllLinkTypes && linkTypeRoleTuples.length > 0) && isPreferenceSpecified) {
				// 100412: add separator because none is added after last link type
				linkTypeOptions.push(separatorOption);
				// preferences are specified, add the More... option
				linkTypeOptions.push({
						label: domutils.getXMLEncodedText(this._messages.moreLinkType),
						value: MORE_TYPE_VALUE
				});
			}
			
			this._linkTypeSelect.addOption(linkTypeOptions);
			if (!skipLinkTypeChanged) {
				this._linkTypeChanged();
			}
			return linkTypeRoleTuples;
		});
		
		return linkTypeRolesDeferred;
	},

	_setProjectPickerToUseAllProjects: function() {
		this._linkArtifactPane.setProjectPickerToUseAllProjects();
	},

	_setProjectPickerToBeRestrictedToProject: function(projectURI) {
		this._linkArtifactPane.setProjectPickerToBeRestrictedToProject(
			projectURI);
		this._targetProject = projectURI;
	},
	
	_enableLinkToWeb: function(enable) {
		// Enable/disable the Link to Web option.
		
		if (enable) {
			this._linkArtifactPane.enableLinkToWeb();
		} else {
			this._linkArtifactPane.disableLinkToWeb();
		}
	},

	resetDialog: function(){
		this.hideMessage();
		dojo.attr(this._cancelButton, "disabled", false);
	},
	
	_updateValues: function(args){
		
		// this function is used to update the values when using the PersistentLinkDialog
		
		this.resetDialog();
		
		this.request = args.request;
		
		this.isApplyLinkConstraints = args.isApplyLinkConstraints;
		if (this.isApplyLinkConstraints) {
			this._crossProjectLinkTypeRoles = {};
		}
		
		// resource has changed
		if(this.resourceInfo != args.resourceInfo || 
		   //this.resourceInfos != args.resourceInfos ||
		   this.resourceURI != args.resourceURI ||
		   this.resourceURIs != args.resourceURIS || 
		   this.resourceName != args.resourceName){
			
			// update values
			this.resourceInfo 	= args.resourceInfo; 
			this.resourceInfos 	= args.resourceInfos;
			this.resourceURI 	= args.resourceURI;
			this.resourceURIs 	= args.resourceURIs;
			this.resourceName 	= args.resourceName;

			this._setupResourceInfo();
			
		}
		
		
		// reset link types

		if(this.artifactTypeURI !== args.artifactTypeURI 
				|| this.allowNoLinkType !== args.allowNoLinkType
				|| this.includeCLM !== args.includeCLM
				|| this.linkTypeWhitelist !== args.linkTypeWhitelist
				|| this.isCollection !== args.isCollection
				|| this.forcePreferredLinkTypes !== args.forcePreferredLinkTypes){
			

			this.artifactTypeURI 			= args.artifactTypeURI; 
			this.allowNoLinkType 			= args.allowNoLinkType;
			this.includeCLM 				= args.includeCLM;
			this.linkTypeWhitelist 			= args.linkTypeWhitelist;
			this.isCollection 				= args.isCollection;
			this.forcePreferredLinkTypes 	= args.forcePreferredLinkTypes;

		}
		
		
		// empty the links type drop down
		try{
			this._linkTypeSelect.removeOption(this._linkTypeSelect.getOptions());
		}catch(e){}
		
		// set variables used by _initLinkTypes
		this.linkTypeURI = args.linkTypeURI ? args.linkTypeURI : null;
		this._showAllLinkTypes = false;
		
		// reinit link types
		var linkTypesResetDeferred = this._initLinkTypes(true);
		
		linkTypesResetDeferred.addCallback(dojo.hitch(this, function(result){

			// select new link type
			var buildLinkTypeURI = dojo.hitch(this, function(linkTypeURI, isObject){
				var linkType = this._getLinkTypeFromURI(linkTypeURI);
				if (!linkType){
					return linkTypeURI;
				} else if (linkType.isBidirectional()){
					return linkTypeURI + "/bidirectional";
				} else if (isObject){
					return linkTypeURI + "/object";
				} else {
					return linkTypeURI + "/subject";
				}
			});
			
			var linktype = null;
			
			if(args.linkTypeURI){
				try{
					linktype = buildLinkTypeURI(args.linkTypeURI, args.resourceIsObject);
				}catch(e){}
			}

			this.linkTypeURI = args.linkTypeURI;
			this.resourceIsObject = args.resourceIsObject;
			try{
				this._linkTypeSelect.set("value", linktype);
				// defect 100430 - error with updating the dom when dialog is not visible
				setTimeout(dojo.hitch(this,function(){this._linkTypeChanged();}), 100);
			}catch(e){}

		}));
		

		// update hypertext value
		if(this._hyperLinkTextField && this._hyperLinkTextField.value !== args.hyperLinkText){
			this._hyperLinkTextField.value = args.hyperLinkText;
		}
		
		// update the callbacks
		this.callback = args.callback ? args.callback : null;
		this.onOpen = args.onOpen ? args.onOpen : function(){};
		this.onClose = args.onClose ? args.onClose : function(){};
		

		if (this._linkArtifactPane) {
			
			// update filtered artifacts
			this.filterArtifacts = args.filterArtifacts ? args.filterArtifacts : [];
			this._linkArtifactPane.updateFilteredArtifacts();
			
			// update recently viewed artifacts list
			this._linkArtifactPane.updateRecentlyViewedArtifacts();
			
			// Update the project list
			this._linkArtifactPane.updateProjectList();
			
		}
		
	}
	
});

})();
