/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.ui.picker.LqeArtifactPicker'); //$NON-NLS-1$

dojo.require('jazz.app.auth'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.common.web.ui.picker", "LqeArtifactPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.picker.BasePicker'); //$NON-NLS-1$
dojo.require('jazz.util.i18n'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.picker.FilterList'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.picker.HeaderList'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.doors.DoorsUtils'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.picker.PickerResult'); //$NON-NLS-1$
dojo.require('jazz.ui.Menu'); //$NON-NLS-1$
dojo.require('jazz.ui.menu.MenuItem'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.picker.SortMenu'); //$NON-NLS-1$
dojo.require('jazz.ui.FilterBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.TabGroup'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.picker.QueryPicker'); //$NON-NLS-1$
dojo.require('com.ibm.relm.explorer.web.ui.internal.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.internal.queryparams.SystemParameterUtil'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.internal.queryparams.paramprompt.ParameterPromptDialog'); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.internal.QueryUtils"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.Site'); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.ConfigurationUIUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.ConfigurationContext"); //$NON-NLS-1$

(function ()
{
	var gator_ = jazz.ui.internal.resourceLinkAggreagtor();
	var html = jazz.util.html;
	var DoorsUtils = com.ibm.relm.common.web.doors.DoorsUtils;
	var PickerResult = com.ibm.relm.common.web.ui.picker.PickerResult;
	var SystemParameterUtil = com.ibm.relm.common.web.ui.internal.queryparams.SystemParameterUtil;
	var Site = com.ibm.relm.common.web.ui.Site;
	var queryUtils = com.ibm.relm.common.web.ui.internal.QueryUtils;

	var DEFAULT_ICON = dojo.moduleUrl('com.ibm.relm.common.web', 'ui/images/document.gif'); //$NON-NLS-1$ //$NON-NLS-2$
	var FILTERED_TYPES_COOKIE = 'relm-picker-filtered-types'; //$NON-NLS-1$

	var REQUIREMENT_COLLECTION_RDF_TYPE = 'http://open-services.net/ns/rm#RequirementCollection'; //$NON-NLS-1$

	var dummy = dojo.create('div'); //$NON-NLS-1$
	function cleanHtml(text) {
		dummy.innerHTML = text;
		return dummy.textContent;
	}

	function compareValues(left, right) {
		if (!left && !right) return 0;
		if (!left) return 1;
		if (!right) return -1;

		if (left.toLowerCase) left = left.toLowerCase();
		if (right.toLowerCase) right = right.toLowerCase();

		if (left === right) return 0;
		if (left < right) return -1;
		return 1;
	}

	function newCompareFunction(propertyId, sortByAscending) {
		return function (left, right) {
			if (!sortByAscending) {
				var tmp = right;
				right = left;
				left = tmp;
			}

			var leftValue = left[propertyId];
			var rightValue = right[propertyId];

			// fallback to text for items missing the requested property
			if (!leftValue && !rightValue) {
				return compareValues(left.text, right.text);
			}

			var compare = compareValues(leftValue, rightValue);
			if (compare !== 0) return compare;

			// Breaks ties with 'text'. Modified time may be better, but 'text' is reliably always there. //$NON-NLS-1$ //$NON-NLS-2$
			return compareValues(left.text, right.text);
		};
	}


	/**
	 * Creates a new LI for a StyledList given an object returned from the DOORS
	 * listOfBaselines API.
	 * @param  {object} baseline a baseline returned from DOORS for a module.
	 * @return {LI} HTML LI element with a label for the given baseline.
	 */
	function baselineListItemFactory(baseline) {

		var result = new PickerResult({
			item: baseline
		});

		// XXX: Only safe since StyledList will use dijit.findWidgets to .destroy the PickerResult
		return result.domNode;
	}

	// For prompting queries, we save the last used values per query URI.
	// This lets us prefill values for the user each time the run it.
	var oldValuesByQueryUri = {};

	/**
	 * Picker dialog for artifacts from the LQE index.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.ui.picker.LqeArtifactPicker', //$NON-NLS-1$
		[com.ibm.relm.common.web.ui.picker.BasePicker], 
		{
			prevXhr: null,
			showAddAndClose: false,
			supportsPreview: true,
			filterProducts: false,
			filterUsers: true,

			/**
			 * Optional array or rdf:types. If given, they type filters will
			 * automatically be set to only return results of these types. Each
			 * type is a full URL to an RDF type like
			 * "http://jazz.net/ns/dm/rhapsody/uml#Class". Optionally, the type
			 * can have a more specific "app type", which is practice is only
			 * used for picking the dcterms:type of change requests, like "http
			 * ://open-services.net/ns/cm#ChangeRequest : Defect". Since this
			 * format could change and the names of the sub types are process
			 * dependent, callers would not normally pass this, but it is allowed
			 * since the underlying filter code suppors this syntax.
			 * 
			 * @type {array} array of strings listing RDF types.
			 */
			requiredArtifactTypes: null,

			constructor: function (args)
			{
				this._messages = dojo.i18n.getLocalization("com.ibm.relm.common.web.ui.picker", "LqeArtifactPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
				this.dialogTitle = this._messages.dialogTitle;
				this.iconClass = 'ple-icon-addArtifact'; //$NON-NLS-1$
				// Hide the search label. The Tab stand in as the labels now
				this.searchLabel = ''; //$NON-NLS-1$
				this.searchTitle = this._messages.resourcePickerHelp;
				this._parentPath = args.parentPath;
				this._parentVersionUri = args.parentUri;
				this._parentPartUri = args.parentPart;
				this.timestamp = new Date().getTime();
				this._sortProperty = 'relevance'; //$NON-NLS-1$

				// by default, it's sorted by relevance. Don't resort in that case. //$NON-NLS-1$
				this._isSorted = false;
				this.requiredArtifactTypes = args.requiredArtifactTypes;
			},

			_initTypeFilterButton: function() {
				var self = this;

				this._typeFilterList = new com.ibm.relm.common.web.ui.picker.FilterList({
					label: this._messages.typesHeader,
					filteredTypesCookieName: FILTERED_TYPES_COOKIE,
					inPopup: true
				});

				dojo.addClass(this._typeFilterList.domNode, 'ple-FilterList-popup'); //$NON-NLS-1$

				var typeFilterMenuItem = new jazz.ui.menu.NodeMenuItem({
					focusable: false,
					_onClick: function(e) {
						e.stopPropagation();
					}
				});
				typeFilterMenuItem.focusNode.style.outline = 'none'; //$NON-NLS-1$
				typeFilterMenuItem.addChild(this._typeFilterList);

				typeFilterMenuItem.connect(typeFilterMenuItem.domNode, 'onkeypress', function(e) { //$NON-NLS-1$
					var key = e.keyCode;

					if (key === dojo.keys.TAB) {
						// The filter button thinks TAB means to move to its peer widget.
						// Normally that's right, but we want to override here so that
						// TAB can navigate controls in the filter popup.
						// Other keys pass though, so ESC will still close the popup.
						e.stopPropagation();
					}
				});

				this._typeFilterMenu = new jazz.ui.Menu({
					onOpen: function() {
						// The filter list can potentially be long and try to scroll off the screen.
						// That's ugly and hard to use, so set a max height for it. 
						// Guess the amount of space we have below the menu 
						var buttonPos = dojo.position(self._typeFilterButton.domNode);
						var spaceBelowButton = dijit.getViewport().h - buttonPos.y - buttonPos.h;
						var fudgeFactor = 110;
						var maxHeight = spaceBelowButton - fudgeFactor;
						self._typeFilterList._toggleArea.style.maxHeight = maxHeight + 'px'; //$NON-NLS-1$

						// Change focus to the type fileter list when it opens.
						// This lets it handle keyboard events. Otherwise, the button
						// would still have focus, and would close the popup on TAB.
						self._typeFilterList.focus();
					}
				});

				this._typeFilterMenu.addChild(typeFilterMenuItem);

				this._typeFilterButton = new jazz.ui.toolbar.MenuButton({
					enabled: false,
					label: this._messages.filterByType,
					labelPressed: this._messages.filterByType,
					imgClass: 'ple-icon-filter', //$NON-NLS-1$
					destroyOnClose: false,
					menuProvider: function() {
						return self._typeFilterMenu;
					}
				});

				this.connect(this._typeFilterList, 'onChange', function() { //$NON-NLS-1$
					this.requiredArtifactTypes = null; // clear requiredArtifactTypes to do ordinal filter in doSearch
					this._doFreshSearch();
					self._typeFilterButton.close();
				});

				this.connect(this._typeFilterList, 'onRevert', function() { //$NON-NLS-1$
					self._typeFilterButton.close();
				});

				dojo.addClass(this._typeFilterButton.domNode, 'r-float-left'); //$NON-NLS-1$
				this._typeFilterButton.placeAt(this.searchOptionsNode);
			},

			/**
			 * Filters the search results.
			 * @param  {string} text text to search for in the current results. Each word is searched for.
			 */
			_filterByText: function(text) {
				this.filterHits(text, ['artifactType', 'contributorId']); //$NON-NLS-1$ //$NON-NLS-2$
			},

			_shouldClearFilter: function() {
				// Remove the dwell time to ensure the results are cleared instantly.
				var savedDwellTime = this._filterBox.dwellTime;
				this._filterBox.dwellTime = 0;
				this._filterBox.clear();
				this._filterBox.dwellTime = savedDwellTime;
			},

			_initFilterBox: function() {
				this._filterBox = new jazz.ui.FilterBox({
					defaultText: this._messages.typeToFilterResults,
					dwellTime: 250,
					onChange: dojo.hitch(this, this._filterByText)
				});
				dojo.addClass(this._filterBox.domNode, 'r-right r-radius ple-LqeArtifactPicker-filterBox'); //$NON-NLS-1$

				this._filterBox.placeAt(this.searchOptionsNode);

				// Call back from "show all" link in hidden results should clear the filter box. //$NON-NLS-1$
				this.connect(this.resultList, 'shouldClearFilter', this._shouldClearFilter); //$NON-NLS-1$

				this.connect(this._filterBox._box, 'onkeyup', this._filterBoxOnKeyUp); //$NON-NLS-1$
			},

			_filterBoxOnKeyUp: function(e) {
				var k = e.keyCode;
				
				if (k === dojo.keys.UP_ARROW) {
					dojo.stopEvent(e);
					this.selectPreviousResult();
					return;
				}
				else if (k === dojo.keys.DOWN_ARROW) {
					dojo.stopEvent(e);
					this.selectNextResult();
					return;
				}
			},

			_initSortButton: function() {
				var properties = [
					['relevance', this._messages.relevanceProperty], //$NON-NLS-1$
					['artifactType', this._messages.artifactTypeProperty], //$NON-NLS-1$
					['contributorName', this._messages.contributorNameProperty], //$NON-NLS-1$
					['createdTime', this._messages.createdTimeProperty, true], //$NON-NLS-1$
					['modifiedTime', this._messages.modifiedTimeProperty, true], //$NON-NLS-1$
					['title', this._messages.titleProperty], //$NON-NLS-1$
					['identifier', this._messages.identifierProperty], //$NON-NLS-1$
					['cmStatus', this._messages.cmStatusProperty] //$NON-NLS-1$
				];

				this._sortMenu = new com.ibm.relm.common.web.ui.picker.SortMenu({
					properties: properties,
					sortOnProperty: 'relevance' //$NON-NLS-1$
				});

				this.connect(this._sortMenu, 'didSortBy', this._sort); //$NON-NLS-1$

				var label = dojo.replace(this._messages.sortByPropertyX, [this._messages.relevanceProperty]);

				var self = this;
				this._sortButton = new jazz.ui.toolbar.MenuButton({
					enabled: true,
					label: label,
					labelPressed: label,
					imgClass: 'ple-icon-sortAscending', //$NON-NLS-1$
					destroyOnClose: false,
					menuProvider: function() {
						return self._sortMenu;
					}
				});
				dojo.addClass(this._sortButton.domNode, 'r-left r-margin-r'); //$NON-NLS-1$

				this._sortButton.placeAt(this.searchOptionsNode);
			},

			_sort: function(propertyId, sortByAscending, propertyLabel) {
				var label = dojo.replace(this._messages.sortByPropertyX, [propertyLabel]);
				this._sortButton.set('label', label); //$NON-NLS-1$
				this._sortButton.set('labelPressed', label); //$NON-NLS-1$
				this._sortButton.set('imgClass', //$NON-NLS-1$
					sortByAscending ? 'ple-icon-sortAscending' : 'ple-icon-sortDescending'); //$NON-NLS-1$ //$NON-NLS-2$

				this.sortHits(newCompareFunction(propertyId, sortByAscending));

				this._isSorted = true;
			},

			_hideSearch: function() {
				this._searchBarNodeHeight = dojo.marginBox(this.searchBarNode).h;
				this.searchBarNode.style.display = 'none'; //$NON-NLS-1$

				this.subtractHeight -= this._searchBarNodeHeight;
				this._resizeHeights();
				this.togglePreview(this._previewVisible, false);

			},

			_showSearch: function() {
				this.searchBarNode.style.display = ''; //$NON-NLS-1$

				if (this._searchBarNodeHeight) {
					this.subtractHeight += this._searchBarNodeHeight;
					this._resizeHeights();
					this.togglePreview(this._previewVisible, false);
				}
			},

			_onHideQueries: function() {
				if (!this._queryPicker) return;
				this._queryPicker.domNode.style.display = 'none'; //$NON-NLS-1$
				dojo.removeClass(this.domNode, 'ple-BasePicker-showFilters'); //$NON-NLS-1$
			},

			_onShowQueries: function() {
				if (!this._queryPicker) {
					this._queryPicker = new com.ibm.relm.common.web.ui.picker.QueryPicker();
					this._queryPicker.placeAt(this._filterWrapper, 'first'); //$NON-NLS-1$

					this.connect(this._queryPicker, 'onPickedQueryUri', this._loadAndRunQuery); //$NON-NLS-1$

					dojo.style(this._queryPicker.domNode,  {
						position: 'absolute', //$NON-NLS-1$
						top: '0', //$NON-NLS-1$
						left: '0', //$NON-NLS-1$
						width: '300px', //$NON-NLS-1$
						height: '100%' //$NON-NLS-1$
					});
				}

				dojo.addClass(this.domNode, 'ple-BasePicker-showFilters'); //$NON-NLS-1$
				this._queryPicker.domNode.style.display = ''; //$NON-NLS-1$
			},

			showQueries: function() {
				if(this._queryTab){
					this._queryTab.activate();
				}
			},

			_initTabs: function() {
				var self = this;
				// we wrap the tabs, and everything above them (only MessageBox)
				// in an extra div specifically to give this area a white background
				// that's set off from the content in the tabs themselves.
				var tabContent = dojo.create('div', { //$NON-NLS-1$
					style: 'background-color: white' //$NON-NLS-1$
				});

				dojo.place(this.messageBox.domNode, tabContent);

				this._tabs = new com.ibm.relm.common.web.TabGroup();
				dojo.addClass(this._tabs.domNode, 'relm-TabGroup-grayTabs'); //$NON-NLS-1$
				this.own(this._tabs);
				this._searchTab = this._tabs.addTab(this._messages.searchTabTitle, {
					hide: dojo.hitch(this, this._hideSearch),
					show: dojo.hitch(this, this._showSearch)
				});
				dojo.when(queryUtils.getQueryOptions(), function(response){
					if(response.enableArtifactPickerByQueries == true){
						self._queryTab = self._tabs.addTab(self._messages.queryTabTitle, {
							hide: dojo.hitch(self, self._onHideQueries),
							show: dojo.hitch(self, self._onShowQueries)
						});
					}
				});

				this._tabs.placeAt(tabContent);
				dojo.place(tabContent, this.domNode, 'first'); //$NON-NLS-1$

				var activeTabKey = 'relm.LqeArtifactPicker.activeTab'; //$NON-NLS-1$
				var activeTabId = window.localStorage[activeTabKey];
				if (activeTabId === 'queries') { //$NON-NLS-1$
					if(this._queryTab){
						this._queryTab.activate();
					}else{
						activeTabId = 'search'; //$NON-NLS-1$
					}
				}

				var self = this;
				this._tabs.watch('activeTab', function(name, oldTab, newTab) { //$NON-NLS-1$
					var activeTabId = newTab === self._searchTab ? 'search' : 'queries'; //$NON-NLS-1$ //$NON-NLS-2$
					window.localStorage[activeTabKey] = activeTabId;
				});
			},

			postCreate: function() {
				this.inherited(arguments);
				this.liveSearchBox.set('placeholder', this._messages.typeToSearch); //$NON-NLS-1$

				this.pageBar.style.display = ''; //$NON-NLS-1$

				dojo.addClass(this.searchOptionsNode, 'ple-LqeArtifactPicker-searchOptions'); //$NON-NLS-1$

				this._initTabs();
				this._initSortButton();
				this._initTypeFilterButton();
				this._initFilterBox();

				if (this.showAddAndClose) {
					this.submitButton.innerHTML = html.escape(this._messages.addAndClose);
					this.submitButton.title = this._messages.addAndCloseTooltip;
					// Force more width, otherwise the button is really skinny in some browsers
					this.submitButton.style.minWidth = '9em'; //$NON-NLS-1$

					this.cancelButton.innerHTML = html.escape(this._messages.close);

					this.addAndStayOpenButton = dojo.create('button', { //$NON-NLS-1$
						type: 'button', //$NON-NLS-1$
						innerHTML: this._messages.ok,
						disabled: true,
						title: this._messages.addTooltip
					});

					this.connect(this.addAndStayOpenButton, 'onclick', this._onAddNow); //$NON-NLS-1$
					dojo.place(this.addAndStayOpenButton, this.submitButton, 'after'); //$NON-NLS-1$

					// Buttons in the templates have newlines between them,
					// which results in a collapsed text node between them,
					// which takes up some space. We simulate that here by
					// forcing in an empty text node to make this button have
					// the same spacing as the others
					dojo.place(document.createTextNode(' '), this.submitButton, 'after'); //$NON-NLS-1$ //$NON-NLS-2$
				}
			},

			_onAddNow: function() {
				if (this.submitButton.disabled) return; 
				var item = this.currentList().getSelectedItem();

				if (item) {
					var stayOpen = true;
					this.onOk(item, stayOpen);
				}
			},

			_selectionChanged: function(item) {
				this.inherited(arguments);

				if (this.addAndStayOpenButton) {
					this.addAndStayOpenButton.disabled = !item ? true : false;
				}
			},

			/**
			 * Override and insert a custom "local" preview for baselines,
			 * since they don't have a real OSLC preview (only for the module).
			 * @param {object} item possibly a DOORS baseline module. Otherwise,
			 * pass to the parent method.
			 */
			setPreviewItem: function(item) {
				if (item && item.isDoorsBaseline) {
					var localPreviewNode = this._oslcPreview.showLocalPreview(
						item.uri,
						html.escape(item.version),
						dojo.moduleUrl('com.ibm.relm.common.web', 'ui/images/doorsBaseline.png')); //$NON-NLS-1$ //$NON-NLS-2$

					dojo.empty(localPreviewNode);
					var baselinePreview = DoorsUtils.createBaselinePreview(item);
					dojo.place(baselinePreview, localPreviewNode);
					return;
				}

				this.inherited(arguments);
			},

			uninitialize: function()
			{
				dojo.disconnect(this._moreResultsHandle);
				this._typeFilterList && this._typeFilterList.destroyRecursive();
				this._typeFilterButton && this._typeFilterButton.destroyRecursive();
				this._typeFilterMenu && this._typeFilterMenu.destroyRecursive();
				this._sortButton && this._sortButton.destroyRecursive();
				this._sortMenu && this._sortMenu.destroyRecursive();
				this._filterBox && this._filterBox.destroyRecursive();
				this.inherited(arguments);
			},
			
			_getXhrParams: function(resource) {
				var site = this.selectedSite;

				var params = {
					toSite: Site.siteToJsonSite(site),
					artifactUri: resource.uri,
					artifactConceptUri: resource.conceptUri,
					artifactTitle: resource.text
				};

				if (resource.isDoorsBaseline) {
					return dojo.mixin(params, DoorsUtils.saveLinkArgs(resource));
				} else {
					return params;
				}
			},

			onOk: function(resource, stayOpen) {
				this.messageBox.hideMessage();

				if (this.addAndStayOpenButton) {
					this.addAndStayOpenButton.disabled = true;	
				}

				this.submitButton.disabled = this.cancelButton.disabled = true;
				this.columnBrowser.topWidget().set('enabled', false); //$NON-NLS-1$
				this._typeFilterList.set('enabled', false); //$NON-NLS-1$

				var content = this._getXhrParams(resource);

				var message = com.ibm.relm.common.web.StatusMessage.show(this._messages.addingLink);

				var self = this;
				jazz.client.xhrPost({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/links', //$NON-NLS-1$
					content: content,

					handle: function() {
						message.hide();
					},

					load: dojo.hitch(this, function(pathSegmentItem) {
						var linkPath = self.selectedSite.path.concat(pathSegmentItem.pathSegment);

						com.ibm.relm.common.web.ui.tree.TreeRefresh.refresh({
							selectParentPath: linkPath,
							flashPath: true
						}).addBoth(function() {
							if (stayOpen) {
								self.addAndStayOpenButton.disabled = self.submitButton.disabled = self.cancelButton.disabled = false;
								self.resultList.set('enabled', true); //$NON-NLS-1$
								self._typeFilterList.set('enabled', true); //$NON-NLS-1$
							}
							else {
								self.close();
							}
						});
					}),

					error: function(error, ioArgs) {
						self.resultList.set('enabled', true); //$NON-NLS-1$
						self._typeFilterList.set('enabled', true); //$NON-NLS-1$
						self.submitButton.disabled = self.cancelButton.disabled = false;
						if (self.addAndStayOpenButton) self.addAndStayOpenButton.disabled = self.cancelButton.disabled = false;
						self.messageBox.showXhrError(ioArgs.xhr);
					}
				});

			},
			
			addLink: function(args) {
				// toItemVersionUri, partUri, itemVersionPath, artifactUrl, artifactTitle, callback, errback) 
				
			},

			/**
			 * Loads the details of a query from its URI, then runs that query.
			 * Note that the SPARQL and other query details are not persisted since
			 * we always want to run the latest version (if saved in another window)
			 * with needing to reopen this dialog.
			 * @param  {[type]} queryUri [description]
			 * @return {[type]}          [description]
			 */
			_loadAndRunQuery: function(queryUri) {
				var self = this;

				// Don't clear the results yet. If this is a prompting query, we
				// want be able to cancel it and still see the old results.
				var message = com.ibm.relm.common.web.StatusMessage.show(this._messages.loadingQueryStatusMessage);

				jazz.client.xhrGet({
					url: queryUri,
					headers: { 'Accept' : 'application/json' }, //$NON-NLS-1$ //$NON-NLS-2$
					handleAs: 'json', //$NON-NLS-1$
				
					handle: function() {
						message.hide();
					},

					load: function (rdf) {
						// "default" RDF resource is our query info //$NON-NLS-1$
						var queryInfo = rdf['']; //$NON-NLS-1$
						var ns = com.ibm.relm.explorer.web.ui.namespace;
						var sparql = queryInfo[ns.query.sparql][0].value;
						var queryTitle = queryInfo[ns.dcterms.title][0].value;

						// Called finally run the query, either right away or after prompting
						// for params.
						function actuallyDoTheQuery(userParameterValues) {
							var systemParameterValues = com.ibm.relm.common.web.ui.internal.queryparams.SystemParameterUtil.getSystemParameterValues();
							dojo.mixin(userParameterValues, systemParameterValues);
							// Save this for paging results. In that case, we don't want the fresh
							// sparql, since it might change the results. Just use the last executed sparql.
							self._lastSparql = sparql;
							self._lastUserParameterValues = userParameterValues;
							oldValuesByQueryUri[queryUri] = userParameterValues;

							self.willDoSearch();
							self.doSearch(sparql, false, true, userParameterValues);
						}

						var isParameterized = queryInfo[ns.query.parameterized];

						if (!isParameterized || isParameterized[0].value !== 'true') { //$NON-NLS-1$
							actuallyDoTheQuery();
							return;
						}

						var rawParameters = queryInfo[ns.query.parameter];
						var parameters = rawParameters.length ? SystemParameterUtil.getParametersFromQueryInfo(rdf, rawParameters) : null;
						
						var oldValues = oldValuesByQueryUri[queryUri];
						self.promptForQueryParameters(queryUri, queryTitle, parameters, oldValues).then(actuallyDoTheQuery);
					},
				
					error: function(error, ioArgs) {
						// This test is needed since an xhr cancel calls the error callback. 
						if (error.dojoType === 'cancel') { return; } //$NON-NLS-1$
						if (!ioArgs.xhr.responseText) { return; }

						self.resultList.clearResults();
						self.clearPreview();
						self.didClearResults();
						self.messageBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			promptForQueryParameters: function(queryUri, queryName, parameters, oldValues) {
				oldValues = oldValues || [];
				var deferred = new dojo.Deferred();

				var promptArgs = {
					needAllParameterValues: true,
					parameters: parameters,
					parameterValues: oldValues,
					queryName: queryName,
					queryUri: queryUri,
					isDirty: true,
					_onCancel: function() {
						deferred.cancel();
					},
					_onSubmit: function(parameterValues) {
						deferred.callback(parameterValues);
					}
				};

				var d = new com.ibm.relm.common.web.ui.internal.queryparams.paramprompt.ParameterPromptDialog(promptArgs);
				d.startup();

				return deferred;
			},

			_doPageChangeSearch: function() {
				var activeTab = this._tabs.get('activeTab'); //$NON-NLS-1$

				this.willDoSearch();
				var isPageChange = true;

				if (activeTab === this._queryTab) {
					if (this._lastSparql) {
						var isSparqlQuery = true;
						this.doSearch(this._lastSparql, isPageChange, isSparqlQuery, this._lastUserParameterValues);
					}
				}
				else {
					this.doSearch(this.liveSearchBox.getSearchString(), isPageChange);
				}
			},

			_doFreshSearch: function() {
				var activeTab = this._tabs.get('activeTab'); //$NON-NLS-1$

				if (activeTab === this._queryTab) {
					if (this._lastSparql) {
						this.willDoSearch();
						var isPageChange = false;
						var isSparqlQuery = true;
						this.doSearch(this._lastSparql, isPageChange, isSparqlQuery, this._lastUserParameterValues);
					}
				}
				else {
					this.liveSearchBox.runSearch();	
				}
			},

			doSearch: function(searchString, isPageChange, isSparqlQuery, parameterValues) {
				if (!isPageChange) {
					this.pageNumber = 1;
					// update the timestamp for new searches, not page changes.
					// This allows us to using caching while paging results, but
					// break it for new searches.
					this.timestamp = new Date().getTime();
				}

				var filteredTypes;
				var typeFilterStyle;

				if (dojo.isArray(this.requiredArtifactTypes) && this.requiredArtifactTypes.length > 0) {
					typeFilterStyle = 'include'; //$NON-NLS-1$
					filteredTypes = this.requiredArtifactTypes;
				}
				else {
					typeFilterStyle = 'exclude'; //$NON-NLS-1$

					// Restore saved filter if there are any. We won't show them
					// until after the search run, but the will still affect the results.
					// We don't want to show filter for types that aren't found //$NON-NLS-1$
					// by the search, so we'll only show the bar after the search runs.
					filteredTypes = this._typeFilterList.mergeCurrentAndSavedFilters();

					// persist this for the next time the dialog shows
					window.localStorage[FILTERED_TYPES_COOKIE] = dojo.toJson(filteredTypes);
				}

				if (this.prevXhr) {
					this.prevXhr.cancel();
				}

				var xhrContent = {
					filteredTypes: dojo.toJson(filteredTypes),
					filterProducts: this.filterProducts,
					filterUsers: this.filterUsers,
					decorateProductTitles: true,
					pageSize: this.getSelectedPageSize(),
					pageNumber: this.pageNumber,
					timestamp: this.timestamp,
					typeFilterStyle: typeFilterStyle 
				};

				if (isSparqlQuery) {
					var method = 'POST'; //$NON-NLS-1$
					xhrContent.sparql = searchString;

					if (parameterValues) {
						xhrContent.parameterValues = dojo.toJson(parameterValues);
					}
				}
				else {
					method = 'GET'; //$NON-NLS-1$
					xhrContent.q = searchString;
				}

				var self = this;
				self.messageBox.hide();
				this.prevXhr = jazz.client.xhr(
					method, {
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/lqe_full_text_search', //$NON-NLS-1$
					content: xhrContent,

					load: function(result) {
						var hits = result.hits;

						//Need to do explicit compare to undefined since 0 is a valid number
						if (result.totalCount !== undefined && !isPageChange) {
							self._pager.startPage = 0;
							self._pager.setTotal(result.totalCount, result.isFuzzyTotal);
						}

						if (isPageChange) {
							var startPageOffset = Math.floor(self._pager._start / self._pager._getPageSize());
							self._pager.startPage = startPageOffset;
							self._pager.setTotal(result.totalCount, result.isFuzzyTotal);	
							self._typeFilterList.set('enabled', true); //$NON-NLS-1$
							self._updateTypeFilterCounts();
						}
						else {
							self._showFilters(result.types);
							self.doLoadTypes(searchString, isSparqlQuery, parameterValues);
						}

						if (self._isSorted) {
							var sortOnProperty = self._sortMenu.get('sortOnProperty'); //$NON-NLS-1$
							var sortByAscending = self._sortMenu.get('sortByAscending'); //$NON-NLS-1$
							hits.sort(newCompareFunction(sortOnProperty, sortByAscending));
						}
						self.showResults(hits);
					},
					
					error: function(error, ioArgs) {
						// This test is needed since an xhr cancel calls the error callback. 
						if (error.dojoType === 'cancel') { return; } //$NON-NLS-1$
						if (!ioArgs.xhr.responseText) { return; }

						self.messageBox.hideMessage();
						self.resultList.clearResults();
						self.clearPreview();
						self.didClearResults();
						self.messageBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			_updateTypeFilterCounts: function() {
				var counts = this._typeFilterList.getCounts();

				var hasSomeTypes = !!counts.total;

				if (!hasSomeTypes) {
					var typeFilterlabel	= this._messages.filterByType;
					this._typeFilterButton.set('enabled', false); //$NON-NLS-1$
					this._typeFilterButton.close();
				}
				else {
					this._typeFilterButton.set('enabled', true); //$NON-NLS-1$
					typeFilterlabel = dojo.replace(this._messages.filterByTypeShowingXOfX, [counts.enabled, counts.total]);
				}

				this._typeFilterButton.set('label', typeFilterlabel); //$NON-NLS-1$
				this._typeFilterButton.set('labelPressed', typeFilterlabel); //$NON-NLS-1$

				var allTypesAreHidden = counts.enabled === 0 && counts.total > 0;
				dojo.toggleClass(this._typeFilterButton.domNode, 'ple-LqeArtifactPicker-warning', allTypesAreHidden); //$NON-NLS-1$
				dojo.toggleClass(this.domNode, 'ple-LqeArtifactPicker-allTypesHidden', allTypesAreHidden); //$NON-NLS-1$

				if (!counts.enabledLabels.length) {
					this.resultList.setHeaderContent(null);
				}
				else {
					var headerMessage = dojo.replace(this._messages.followingTypesXAreVisible, [counts.enabledLabels.join(', ')]); //$NON-NLS-1$
					var header = dojo.create('div', { //$NON-NLS-1$
						className: 'ple-LqeArtifactPicker-resultsHeader j-hint j-small', //$NON-NLS-1$
						innerHTML: headerMessage
					});
					this.resultList.setHeaderContent(header);
				}

				if (!counts.disabledLabels.length) {
					this.resultList.setFooterContent(null);
				}
				else {
					var footerMessage = dojo.replace(this._messages.followingTypesXAreHidden, [counts.disabledLabels.join(', ')]); //$NON-NLS-1$
					var footer = dojo.create('div', { //$NON-NLS-1$
						className: 'ple-LqeArtifactPicker-resultsFooter j-hint j-small', //$NON-NLS-1$
						innerHTML: footerMessage
					});

					this.resultList.setFooterContent(footer);
				}
			},

			// Copied from SearchResults. Should have better code sharing.
			doLoadTypes: function(searchString, isSparqlQuery, parameterValues) {
				this._loadTypesXhr && this._loadTypesXhr.cancel();

				var filteredTypes = null;
				var typeFilterStyle;

				if (dojo.isArray(this.requiredArtifactTypes) && this.requiredArtifactTypes.length > 0){
					filteredTypes = this.requiredArtifactTypes;
					typeFilterStyle = 'include'; //$NON-NLS-1$
				}
				else {
					typeFilterStyle = 'exclude'; //$NON-NLS-1$
					filteredTypes = this._typeFilterList.mergeCurrentAndSavedFilters();
				}

				this._typeFilterList.toggleUpdating(true);

				var xhrContent = {
					typesOnly: true,
					pageNumber: this.pageNumber,
					filteredTypes: dojo.toJson(filteredTypes),
					filterProducts: this.filterProducts,
					filterUsers: this.filterUsers,
					timestamp: this.timestamp,
					typeFilterStyle: typeFilterStyle
				};

				if (isSparqlQuery) {
					var method = 'POST'; //$NON-NLS-1$
					xhrContent.sparql = searchString;

					if (parameterValues) {
						xhrContent.parameterValues = dojo.toJson(parameterValues);
					}
				}
				else {
					method = 'GET'; //$NON-NLS-1$
					xhrContent.q = searchString;
				}

				var self = this;
				this._loadTypesXhr = jazz.client.xhr(method, {
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/lqe_full_text_search', //$NON-NLS-1$
					content: xhrContent,

					handle: function() {
						self._typeFilterList.toggleUpdating(false);
						self._typeFilterList.set('enabled', true); //$NON-NLS-1$
					},

					load: function(response) {
						var allTypes = (response && response[""]) || []; //$NON-NLS-1$
						self._typeFilterList.updateTokens(allTypes);
						self._updateTypeFilterCounts();
					},

					error: function(error, ioArgs) {
						console.error(error, ioArgs);
					}
				});
			},

			willDoSearch: function() {
				this.inherited(arguments);
				this._sortMenu.enableAll();
				this._typeFilterList.set('enabled', false); //$NON-NLS-1$
			},

			willShowResults: function() {
				dojo.disconnect(this._moreResultsHandle);
				window.clearInterval(this._linkRefreshTimeout);
				this._typeFilterList.set('enabled', true); //$NON-NLS-1$
			},

			_showAllResults: function(e) {
				dojo.stopEvent(e);
				this.runSearch(true);
			},

			createListItem: function(item) {
				var result = new PickerResult({
					item: item
				});

				var context = com.ibm.relm.common.web.ui.ConfigurationContext.getGlobalConfigurationContext();
				var itemUri = com.ibm.relm.common.web.ui.ConfigurationUIUtils.getConfigurationResourceUri(item.conceptUri || item.uri, context ? context.configuration : null);
				gator_.register(itemUri, dojo.hitch(result, result.updateFromOlscPreview), function() {
					// ignore failures
				});

				if (item.isBaselined) {
					dojo.create('img', { //$NON-NLS-1$
						src: this._blankGif,
						className: 'ple-icon-isOverlay ple-icon-baselineOverlay' //$NON-NLS-1$
					}, result.titleBlock);
				}

				if (item.isDoors9Module && DoorsUtils.enabled && !this.hideBaselines) {
					var showBaselinesLink = result.addDetailsLink(this._messages.viewBaselines);
					this.connect(showBaselinesLink, 'click', this._showBaselines); //$NON-NLS-1$
				}

				// XXX: Only safe since StyledList will use dijit.findWidgets to .destroy the PickerResult
				return result.domNode;
			},

			/**
			 * Called with the "Show Baselines" link for a DOORS module is clicked. Slides
			 * over the results to show the baseline for the selected module.
			 * 
			 * @param  {even} e the click event.
			 */
			_showBaselines: function(e) {
				dojo.stopEvent(e);

				var row = dojo.NodeList(e.target).closest('li')[0]; //$NON-NLS-1$

				var backLabel = ''; //$NON-NLS-1$

				var selectedItem = null;
				
				if (row) {
					this.resultList.selectRow(row);
					selectedItem = this.resultList.getItemForRow(row);
					backLabel = selectedItem.text;
				}

				// Clear the preview of the module if there
				this.clearPreview();

				var headerList = new com.ibm.relm.common.web.ui.picker.HeaderList({
					backLabel: backLabel
				});

				var baselineList = headerList.list;
				baselineList.set('placeholder', this._messages.noBaselines); //$NON-NLS-1$

				// Register normal event listeners. Then will need to be aware that there are different list levels
				this.connect(baselineList, 'onSelectionChanged', this._selectionChanged); //$NON-NLS-1$
				this.connect(baselineList, 'onDoubleClickItem', this._onOk); //$NON-NLS-1$
				this.connect(baselineList, 'onSubmit', this._onOk); //$NON-NLS-1$
				this.connect(baselineList, 'shouldClearFilter', this._shouldClearFilter); //$NON-NLS-1$

				var self = this;
				baselineList.setListItemFactory(baselineListItemFactory);

				headerList.connect(headerList, 'onBack', function() { //$NON-NLS-1$
					self._typeFilterButton.set('enabled', true); //$NON-NLS-1$
					self.setPagingControlsVisible(true);
					self.columnBrowser.popWidget().then(function(deadWidget) {
							deadWidget.destroyRecursive();
							// Set this after the animation since loading causes the animation to stutter.
							self.setPreviewItem(selectedItem);
							self._filterBox.setValue(self._topLevelFilterBoxValue);
							self._sortMenu.enableAll();
						}
					);
				});

				// Preserve the current filter text since we use the same control
				// when showing baselines. We'll clear the box and restore it later
				// when the user pops back to the top-level list.
				this._topLevelFilterBoxValue = this._filterBox.get('value'); //$NON-NLS-1$
				baselineList.showLoading(true);
				this.columnBrowser.pushWidget(headerList).then(function() {
					self._sortMenu.disableProperties(['artifactType', 'modifiedTime', 'cmStatus']); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					self._filterBox.clear();
				});

				this.setPagingControlsVisible(false);
				self._typeFilterButton.set('enabled', false); //$NON-NLS-1$
				this._loadBaselinesForModule(selectedItem, baselineList);
			},

			/**
			 * Does the XHR load of the baselines for a DOORS module.
			 * May show No Results if there are none.
			 * 
			 * @param  {object} doorsModule the JS object describing the module. An "item".
			 * @param  {widget} styledList  the StyledList widget to display the baselines in once they
			 *                              are loaded. Since this "sub list" is created on the fly when
			 *                              Show Baselines is clicked, this method cannot hard code a reference to
			 *                              the list widget.
			 */
			_loadBaselinesForModule: function(doorsModule, styledList) {
				var baselines = [];

				var doorsModuleUrl = doorsModule.conceptUri;

				// The request to load the baselines must be formatted as XML. It's all boilerplate
				// except the URI to the module, so load from a template.
				var xmlRequest = dojo.cache('com.ibm.relm.common.web', 'ui/picker/requests/doorsListOfBaselinesForModuleX.xml'); //$NON-NLS-1$ //$NON-NLS-2$
				xmlRequest = dojo.replace(xmlRequest, [doorsModuleUrl]);

				// Need to figure out where the DOORS REST API lives based on the
				// selected requirement. Get the base URL for DOORS server by stripping
				// off the requirement-specific URN bit and everything after.
				var doorsBaseUrl = doorsModuleUrl.replace(/\/urn:.*/, ''); //$NON-NLS-1$

				this.submitButton.disabled = this.cancelButton.disabled = true;

				var self = this;
				jazz.client.xhrPut({
					url: doorsBaseUrl + '/dxl/listOfBaselines', //$NON-NLS-1$
					headers: {
						'Accept': 'application/rdf+xml', //$NON-NLS-1$ //$NON-NLS-2$
						'Content-Type': 'application/rdf+xml' //$NON-NLS-1$ //$NON-NLS-2$
					},
					handleAs: 'xml', //$NON-NLS-1$

					putData: xmlRequest,
				
					load: function (resultDoc) {
						self.messageBox.hideMessage();

						var baselines = [];

						var resultNode = resultDoc.getElementsByTagNameNS('http://jazz.net/doors/xmlns/prod/jazz/doors/2.0/', 'result')[0]; //$NON-NLS-1$ //$NON-NLS-2$
						if (resultNode) {
							var json = resultNode.textContent;
							baselines = dojo.fromJson(json).result;
						}

						var relevance = 0;

						dojo.forEach(baselines, function(baseline) {
							// convert from secs to millis
							baseline.date *= 1000;
							baseline.isDoorsBaseline = true;
							baseline.module = doorsModuleUrl;
							baseline.moduleName = doorsModule.text;

							// These baselines are from DOORS and use different property names.
							// Change them to match what RELM searches return, as those names are assumed
							// by PickerResult.
							baseline.relevance = relevance++;
							baseline.text = DoorsUtils.getBaselineTitleHtml(baseline);
							baseline.createdTime = baseline.date;
							baseline.contributorName = baseline.user;
							baseline.iconClass = 'ple-icon ple-icon-doorBaseline'; //$NON-NLS-1$
							// OK, version is not ID, but there's no exact match. It's close. //$NON-NLS-1$
							baseline.id = baseline.version;
						});

						styledList.showResults(baselines);
					},
				
					handle: function() {
						self.submitButton.disabled = self.cancelButton.disabled = false;
					},

					error: function(error, ioArgs) {
						styledList.clearResults();
						
						var errorObject = DoorsUtils.couldNotLoadBaselines(error, ioArgs);
						self.messageBox.showErrorObject(errorObject, true);

						// XXX: This timeout is to prevent overlaping animations of sliding in
						// and out if the user is clicking "Show Baselines" quickly. //$NON-NLS-1$
						// It's not perfect, but since this only comes up if there is a DOORS
						// connection error, it shouldn't be the common case.
						setTimeout(function() {
							self.columnBrowser.topWidget().onBack();
						}, 400);
					}
				});			
			},

			_updateIconAndTitle: function(iconNode, titleNode, result) {
				// no inline oauth login here
				if (result.oauthRequired) return;

				var title;
				var icon;
				var preview;

				if (result.oslcCompact) {
					title = result.oslcCompact.title || result.oslcCompact.shortTitle;
					icon = result.oslcCompact.icon;
				}
				else if (result.compactRendering) {
					title = result.compactRendering.title || result.compactRendering.shortTitle;
					icon = result.compactRendering.icon;
				}
				else {
					console.debug('Do not know how to render this preview style:', result); //$NON-NLS-1$
					return;
				}

				if (icon) {
					dojo.attr(iconNode, 'src', icon); //$NON-NLS-1$
				}

				if (title) {
					// I expect this to be escaped HTML already.
					titleNode.innerHTML = title;
				}
			},


			didClearResults: function() {
				window.clearInterval(this._linkRefreshTimeout);
				this._typeFilterList.clearTokens();
				this._showFilters([]);
			},

			didShowResults: function() {
				this._filterBox._fireOnChange();
			},

			_showFilters: function(types)
			{
				this._typeFilterList.clearTokens();

				for (var i = 0; i < types.length; i++)
				{
					var type = types[i];
					this._typeFilterList.addToken(type.label, type.value, type.iconSrc, type.enabled);
				}
				this._updateTypeFilterCounts();
			}
		}
	);

	com.ibm.relm.common.web.ui.picker.LqeArtifactPicker.baselineListItemFactory = baselineListItemFactory;
	
})();
