/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.layout.AbstractLayout"); //$NON-NLS-1$

// Just for Debug
// dojo.config.debugAtAllCosts = true;

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.QueryBuilderServiceUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.QueryBuilderUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$

// for debug
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.router.ManhattanConnectionRouter"); //$NON-NLS-1$

dojo.requireLocalization('com.ibm.relm.explorer.web.ui.internal', 'DiagramPageMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
var JazzUIWorkbench = com.ibm.relm.explorer.web.ui.internal.util.getJazzUIWorkbench();
var utils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;
var qbUtils = com.ibm.relm.explorer.web.ui.internal.view.nodes.QueryBuilderUtils;
var leServiceUtils = com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils;
var qbServiceUtils = com.ibm.relm.le.ui.internal.QueryBuilderServiceUtils;
var DEFAULT_COMPONENT_TYPE = "baseResourceNode"; //$NON-NLS-1$
var EMPTY_RESULT = {head: {vars: []}, results: {bindings: []}, resultuuid: "empty", sparql: "empty", totalSize: 0}; //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.layout.AbstractLayout", //$NON-NLS-1$
	null,
{
	_container: null,

	constructor: function(args){
		dojo.mixin(this, args);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramPageMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
	},

	/*
	 * subclass should implement
	 */
	setup: function(){
	},
	
	/*
	 * subclass should implement
	 */
	layout: function(){
	},
	
	/*
	 * subclass should implement
	 */
	getLayoutManager: function(domNode){
		return null;
	},

	/*
	 * reusable utility methods for subclass
	 */
	createQueryChildren: function(){
		var self = this;
		var viewer = this._container._viewer;
		var deferred = new dojo.Deferred();
		dojo.when(viewer.inCompoundCommand, function(){
			dojo.when(self._container.getQueries(), function(queries) {
				if(queries){
					dojo.when(self.createChildrenFromQueries(queries), function() {
						deferred.callback();
					});
				} else {
					deferred.callback();
				}
			});
		});
		viewer.queueLoadResource(deferred);
	},
	
	createChildrenFromQueries: function(queries){
		return this.executeQuery(queries, this.createChildrenFromQueryResult);
	},

	executeQuery: function(queries, callback, errorback, dontShowLoadingIcon){
		var queryArray = dojo.isArray(queries) ? queries : [queries];
		
		var counter = {count: 0, total: queryArray.length};
		var container = this._container;
		var viewer = container._viewer;
		var self = this;

		if (container.resourceLoading) 
			container.resourceLoading();

		var xhrArray = [];
		dojo.forEach(queryArray, function(query){
			var xhr = null;
			if (container.setupLinkedQuery) {
				xhr = new dojo.Deferred();
				dojo.when(container.setupLinkedQuery(viewer, query), function(query) {
					dojo.when(self._executeQuery(query, callback, errorback, dontShowLoadingIcon, counter), function(response) {
						xhr.callback(response);
					});;
				});
			} else {
				xhr = self._executeQuery(query, callback, errorback, dontShowLoadingIcon, counter);
			}
			if (xhr) xhrArray.push(xhr);
		});
		return new dojo.DeferredList(xhrArray);
	},

	_executeQuery: function(query, callback, errorback, dontShowLoadingIcon, counter) {
		var finalDeferred = new dojo.Deferred();
		var container = this._container;
		var viewer = container._viewer;
		var config = viewer.config;
		var queryCacheKey = null;
		var queryParams = null;
		var queryEndpoint = config.endpoint;
		if (query.elementId) {
			queryCacheKey = query.elementId;
			queryParams = {parameters: query.parameters, values: query.values, isDownstream: container.isDownstream()};
		} else {
			queryCacheKey = query.sparql;
			queryParams = query.parameterValues;
		}
		var noLoading = container._isNoLoading();
		var cachedResult = (!noLoading && viewer._getCachedResult) ? viewer._getCachedResult(queryCacheKey, queryParams, queryEndpoint) : null;
		var error = dojo.hitch(this, function(response, ioargs){
			if (counter) counter.count++;
			JazzUIWorkbench.hideMessageArea();
			var message = ioargs ? com.ibm.relm.explorer.web.ui.internal.util.getXhrErrorMessage(response, ioargs) : response;
			container.setError(true, message);
			if(dojo.isFunction(errorback)){
				if (container.resourceLoaded) {
					container.resourceLoaded(viewer, null, query.linkinfo);
				}
				errorback.call(this, response, ioargs);
			}
			finalDeferred.callback(); // go ahead even if query error happens
		});
		var callcallback = dojo.hitch(this, function(response, ioargs){
			if (counter) counter.count++;
			JazzUIWorkbench.hideMessageArea();
			var resources = dojo.getObject("results.bindings", false, response);//$NON-NLS-1$
			if (!container.originalOrder) {
				resources = this._sortResources(query, resources);
			}
			if(!resources) {
				finalDeferred.callback();
				return;
			}
			if(viewer._addCachedResult){
				viewer._addCachedResult(queryCacheKey, queryParams, queryEndpoint, response);
			}
			if (container.resourceLoaded) {
				container.resourceLoaded(viewer, resources, query.linkinfo);
			}
			var widgetCreator = callback.call(this, resources, query.linkDef, undefined, response.head, counter);
			if(ioargs && ioargs.args && ioargs.args.com_ibm_relm_lqeAuthSecondTry){
				viewer.fireLoadResource(true);
			}
			if(widgetCreator && dojo.isFunction(widgetCreator.then)){
				widgetCreator.then(finalDeferred.callback, finalDeferred.errback);
			}else{
				finalDeferred.callback();
			}
		});
		var xhr;
		if(cachedResult){
			xhr = new dojo.Deferred();
			xhr.then(callcallback);
			setTimeout(function(){xhr.callback(cachedResult); }, 10);
		}else if(noLoading){
			xhr = new dojo.Deferred();
			xhr.then(callcallback);
			setTimeout(function(){xhr.callback(EMPTY_RESULT);}, 10);
		}else if (query.elementId){
			if (query.isDownstream && (!query.values || query.values.length == 0)) {
				xhr = new dojo.Deferred();
				xhr.then(callcallback);
				setTimeout(function(){xhr.callback(EMPTY_RESULT);}, 10);
			} else {
				JazzUIWorkbench.showMessage(this._messages.runningQuery);
				if(!dontShowLoadingIcon) container.setLoading(true);
				if (query.elementType == 'standard') { //$NON-NLS-1$
					xhr = qbServiceUtils.executeQuery(query.parameters["qbOutput"], function(response) { //$NON-NLS-1$
						qbUtils.convertQueryResult(container, dojo.getObject("results.bindings", false, response));
						callcallback(response); 
					}, error, queryEndpoint, container);
				} else{
					xhr = leServiceUtils.executeQueryById(query.elementId, query.values, query.parameters, callcallback, error, queryEndpoint);
				}
			}
		}else{
			JazzUIWorkbench.showMessage(this._messages.runningQuery);
			var baseQueryUrl = com.ibm.relm.explorer.web.ui.queryServiceUri;
			container.setError(false);
			if(!dontShowLoadingIcon) container.setLoading(true);
			var content ={
				category: "children", //$NON-NLS-1$
				sparql: query.sparql,
				format: false,
				nosparqninresponse: true
			};
			if(queryEndpoint){
				dojo.mixin(content, {endpoint: queryEndpoint});
			}
			if(query.parameterValues){
				content.parameterValues = dojo.toJson(query.parameterValues);
			}

			xhr = viewer.requestXhrPost({
				url: baseQueryUrl,
				headers: {"Accept":"application/json"}, //$NON-NLS-1$ //$NON-NLS-2$
				content: content, 
				load: callcallback,
				error: error,
				handleAs: "json" //$NON-NLS-1$
			});
		}
		return finalDeferred;
	},

	_sortResources: function(query, resources) {
		if (!query || !dojo.isArray(query.values) || !dojo.isArray(resources))
			return resources;
	
		// Avoid sorting cached resources in edit mode 
		var container = this._container;
		if (container && container._getEditMode())  
			resources = resources.slice(0);
		
		// sort resources by order of given values
		var values = query.values;
		resources.sort(function(r1, r2) {
			var weight1 = r1.resource ? dojo.indexOf(values, r1.resource.value) : -1;
			var weight2 = r2.resource ? dojo.indexOf(values, r2.resource.value) : -1;
			return weight1 - weight2;
		});
		return resources;
	},
	
	createChildrenFromQueryResult: function(resources, linkDef, delayed, head, counter){
		var container = this._container;
		if(!container || !container.domNode){
			// maybe the container was destroyed during loading
			return;
		}
		container.setLoading(false);

		var self = this;
		var _prevNumChildren = self._numChildren;
		self._numChildren = resources.length;
		// some layout implementations (e.g. TreeLayout) may assume addResource is called
		// even if no resources are loaded to trigger their functions.
		return this.addResources(container, resources, linkDef).then(function() {
		    // Call layout this method is called first time even when there is no child resources. Otherwise, layout won't occur.
			if(!(_prevNumChildren === 0 && self._numChildren === 0)) {
				// call ancestors layout
				if(delayed === false) delayed = -1;
				else if (delayed === undefined) delayed = container._viewer._delayedLayoutTimeout;
				for(var n = container; n; n = n.getParent()){
					if(delayed >= 0 && dojo.isFunction(n.requestLayout)){
						n.requestLayout(delayed);
					}else if(dojo.isFunction(n.layout)){
						n.layout();
					}
					if(!dojo.isFunction(n.getParent)) break;
				}
			}

			if (!counter || counter.count >= counter.total) {
				self._onAllResourceAdded();
			}
		});
	},
	
	addResources: function(node, resources, linkDef){
		var deferred = new dojo.Deferred();

		var container = this._container;
		var viewer = container._viewer;
		var config = viewer.config;

		// We know that .addChildResource might instantiate some new JS classes.
		// Gather all their names before then and ensure they are loaded before calling down there.
		// Super ugly. I'm (Jay) not convinced this is needed, but I don't know where else ensures that
		// these classes were loaded.
		var componentTypeClassNames = [];
		dojo.forEach(resources, function(resource) {
			var resourceInfo = config.getResourceInfoHasComponent(resource);
			var componentInfo = resourceInfo ? resourceInfo.component : null;
			var componentType = (componentInfo && componentInfo.type) || this.defaultComponentType || DEFAULT_COMPONENT_TYPE;
			var info = viewer.hlp_getComponentInfo(componentType);
			var className = info ? info.className : componentType;
			componentTypeClassNames.push(className);
		}, this);

		var children = container.getChildren();
		var resourceNodeMap = {};
		if (children) {
			for (var i = 0; i < children.length; i++) {
				var res = children[i];
				if (res.uri) resourceNodeMap[res.uri] = res;
			} 
		}
		
		var self = this;
		jazz.core.loader.batch_load_async(componentTypeClassNames, function() {
			var threshold = 100;
			if (resources.length > threshold) {
				self._delayedAddChildResources(node, resources, 0, threshold, linkDef, resourceNodeMap, function() {
					deferred.callback();
				});
			} else {
				self._addChildResources(node, resources, 0, -1, linkDef, resourceNodeMap);
				deferred.callback();
			}
		});

		return deferred;
	},
	
	_delayedAddChildResources: function(node, resources, startIndex, endIndex, linkDef, resourceNodeMap, callback) {
		if (node._destroyed) return;
		
		setTimeout(dojo.hitch(this, function() {
			var cont = this._addChildResources(node, resources, startIndex, endIndex, linkDef, resourceNodeMap);
			if (cont) {
				var batchsize = endIndex - startIndex;
				startIndex = endIndex;
				endIndex = startIndex + batchsize;
				this._delayedAddChildResources(node, resources, startIndex, endIndex, linkDef, resourceNodeMap, callback);
			} else {
				callback();
			}
		}), 10);
	},
	
	_addChildResources: function(node, resources, startIndex, endIndex, linkDef, resourceNodeMap) {
		JazzUIWorkbench.showMessage(this._messages.updatingContainer);
		for (var index = startIndex; index < resources.length; index++) {
			if (endIndex >= 0 && endIndex == index) return true;
			
			var r = resources[index];
			if (r.resource && r.resource.value && resourceNodeMap[r.resource.value]) {
				var resourceNode = resourceNodeMap[r.resource.value];
				// When resources are added to a container which has multiple upstream containers, there might be 
				// the same resources that have been added by the other containers. In this case, a linkDef for each resource 
				// has been already created. So, it is required to update links only. 
				if (resourceNode.updateStaticConnections) {
					// The resourceNode has no linkDef if it has been added by the container which doesn't show connection lines.
					// In this case, linkDef should be added here before updateStaticConnections is called.
					if (!resourceNode.linkDef) {
						var links = this._container.getLinks();
						if(linkDef) links = dojo.mixin(links, linkDef);
						if (links) resourceNode.linkDef = links;
					}
					resourceNode.updateStaticConnections();
				}
				continue;
			}
			var r = resources[index];
			// create resource node
			if(node.reduceRatio) {
				r.reduceRatio = node.reduceRatio;
			}
			// don't allow duplicate entry
			this.addChildResource(node, r, linkDef);
		}
		JazzUIWorkbench.hideMessageArea();
		return false;
	},
	
	addChildResource: function(node, resource, linkDef){
		var container = this._container;
		var viewer = container._viewer;
		var config = viewer.config;

		var resourceInfo = null;
		var componentInfo = null;
		var nodeType = container.getNodeType();
		if(nodeType){
			// if nodeType attribute is specified for the container, use it.
			resourceInfo = config.getResourceInfoById(nodeType);
			if(resourceInfo){
				componentInfo = resourceInfo.component;
			}
		}
		if(!componentInfo){
			// if nodeType is not specified or the comonentInfo for the id doesn't exist
			resourceInfo = config.getResourceInfoHasComponent(resource);
			if(resourceInfo){
				componentInfo = resourceInfo.component;
			}
		}
		var componentType = (componentInfo && componentInfo.type) || this.defaultComponentType || DEFAULT_COMPONENT_TYPE;
		try {
			var links = container.getLinks();
			if(linkDef) links = dojo.mixin(links, linkDef);
			var component = linkDef && linkDef.component;
			var n = viewer.createComponent(componentType, {
				_viewer: viewer,
				resourceInfo: resourceInfo,
				resource: resource,
				linkDef: links,
				linkComponentInfo: component , // for Tree Layout
				hoverOrientation: this._hoverOrientation, // for Tree Layout
				preventSave: true // don't save resources created from query result
			});
			// 1. properties in componentInfo
			dojo.safeMixin(n, utils._loadPropsFromQueryResult(viewer.getAttributes(n), [resource, config.params]));
			// 2. params element under <component> element in resource section
			if(componentInfo && componentInfo.params) {
				// componentInfo.params is formed as [{name: "name1", val: "val1"}, ...],
				// so need to convert to { name1: "val1", ... } by utils.getParamValue
				var conditionEvaluatedParams = utils.evalConditionalParameters(componentInfo.params, resource);
				var compParams = utils.evalParameters(utils.getParamValue(conditionEvaluatedParams), [resource, config.params]);
				dojo.safeMixin(n, compParams);
			}
			// 3. params element under <container> element
			var containerParams = container.paramUses.ui ? container._uiParams : container._params;
			if(containerParams){
				// container._params is formed as { name1: "val1", ... } by NodeContainer._getParams(),
				// so don't need to call utils.getParamValue
				var contParams = utils.evalParameters(containerParams, [resource, config.params]);
				delete contParams.id; // safe
				dojo.safeMixin(n, contParams);
			}
			if(resource.reduceRatio) {
				n.reduceRatio = resource.reduceRatio;
			}
			var uriList = node._uriList;
			var uri = n.uri;
			if(!uri || (uriList && dojo.indexOf(uriList, uri) >= 0)) {
				// don't allow duplicate entry
				n.destroy();
				return null;
			}
			if(dojo.isFunction(node.setLayoutReady))
				node.setLayoutReady(false);
			node.addChild(n);
			return n;
		}catch(e){
			console.error(e);
			throw new Error("Failed to create a component: " + componentType); //$NON-NLS-1$
		}
	}
});

})();
