/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.lpa.app.ui.internal.ContextDetailsWidget"); //$NON-NLS-1$

dojo.require("com.ibm.team.lpa.app.client.LifecycleProjectAdminClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.lpa.common.internal.util"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("jazz.ui.HoverPopup"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.lpa.app.ui.internal", "ContextDetailsWidgetMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var htmlEncode = com.ibm.team.lpa.common.internal.util.htmlEncode;
var ResourceLink = jazz.ui.ResourceLink;
var LifecycleProjectAdminClient = com.ibm.team.lpa.app.client.LifecycleProjectAdminClient;

var TOPIC_DISPLAY_MSG = "com.ibm.team.lpa.app.ui.internal.ProjectsPage#message"; //$NON-NLS-1$

var USES_LINK_TYPES = [
	"implemented-by", //$NON-NLS-1$
	"validates", //$NON-NLS-1$
	"tests", //$NON-NLS-1$
	"tracked-by", //$NON-NLS-1$
	"tracked-by-cm", //$NON-NLS-1$
	"changes-tracked-by", //$NON-NLS-1$
	"elaborates", //$NON-NLS-1$
	"architects", //$NON-NLS-1$
	"validated-by-am" //$NON-NLS-1$
];


var hasDomain = function(lpaProject, domain) {
	var result = false;
	for(var i = 0; i < lpaProject.contexts.length && !result; i++) {
		for (var j = 0; j < lpaProject.contexts[i].domains.length && !result; j++) {
			result = lpaProject.contexts[i].domains[j].uri === domain.uri;
		}		
	}
	return result;
};

dojo.declare("com.ibm.team.lpa.app.ui.internal.ContextDetailsWidget", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
	
	templatePath: dojo.moduleUrl("com.ibm.team.lpa.app","ui/internal/templates/ContextDetailsWidget.html"), //$NON-NLS-1$ //$NON-NLS-2$

	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.lpa.app.ui.internal", "ContextDetailsWidgetMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		
		this.context = args.context;
		this.lpaProject = args.lpaProject;
		
		this.needsRefresh = true;
	},
	
	buildRendering: function() {
		this.inherited(arguments);
	},
	
	postCreate: function() {},
	
	populateServiceProvidersList: function() {
		dojo.forEach(this.links, dojo.hitch(this, function(link){
			var li = dojo.create("li", { //$NON-NLS-1$
				className: "providerItem" //$NON-NLS-1$
			}, this.serviceProvidersList);
			
			var span = dojo.create("span", { //$NON-NLS-1$
				className: "j-label" //$NON-NLS-1$
			}, li);
			
			var content = ""; //$NON-NLS-1$
			var usesLinkIndex = dojo.indexOf(USES_LINK_TYPES, link.linkType);
			
			if (usesLinkIndex > -1){
				//the project uses the functionality
				content = dojo.string.substitute(this.messages.consumesService, {
					serviceName: link.label,
					imageSrc: link.icon ? link.icon : "" //$NON-NLS-1$
				});
			} else {
				//the project provides the functionality
				content = dojo.string.substitute(this.messages.providesService, {
					serviceName: link.label,
					imageSrc: link.icon ? link.icon : "" //$NON-NLS-1$
				});
			}
			
			dojo.attr(span, "innerHTML", content); //$NON-NLS-1$
			
			var rl = new ResourceLink({
				uri: link.serviceProvider.details[0],
				label: jazz.util.html.escape(link.serviceProvider.label),
				lazyFetch: true
			});
			dojo.place(rl.domNode, span);
			
			if (!link.icon) {
				dojo.addClass(dojo.query(".providerImg", span)[0], "hide"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}));
	},
	
	makeVisible: function() {
		if (this.needsRefresh) {
			this.refresh();
			this.needsRefresh = false;
		}
	},
	
	refresh: function(){
		dojo.removeClass(this.loadingMessage, "hide"); //$NON-NLS-1$
		LifecycleProjectAdminClient.getLinks({context: this.context.uri}).addBoth(dojo.hitch(this, function(result){
			dojo.addClass(this.loadingMessage, "hide"); //$NON-NLS-1$
			if (result instanceof Error) {
				var msgType = result.jfsHttpException.msgType;
				
				dojo.publish(TOPIC_DISPLAY_MSG, [{
					content: result.jfsHttpException.message,
					type: (msgType && msgType === "info") ? MessageBox.STATUS_TYPES.INFO : MessageBox.STATUS_TYPES.ERROR //$NON-NLS-1$
				}]);
			}
			else {
				this.links = result;
				this.populateServiceProvidersList();
			}
		}));
	}
});
