/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.lpa.app.ui.internal.ContextPickerDialog"); //$NON-NLS-1$

dojo.require("com.ibm.team.lpa.app.client.LifecycleProjectAdminClient"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._BidiSupport"); //$NON-NLS-1$
dojo.require("jazz.app.i18n");  //$NON-NLS-1$
dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.lpa.app.ui.internal", "ContextPickerDialogMessages"); //$NON-NLS-2$ //$NON-NLS-1$

var LifecycleProjectAdminClient = com.ibm.team.lpa.app.client.LifecycleProjectAdminClient;

//
// Topics
//
var TOPIC_DISPLAY_MSG = "com.ibm.team.lpa.app.ui.internal.ContextPickerDialog#message"; //$NON-NLS-1$


//
// RDF Names
//
var RDF_SYNTAX_TYPE_URI = "http://www.w3.org/1999/02/22-rdf-syntax-ns#type"; //$NON-NLS-1$
var LPA_APPLICATION_URI = "http://jazz.net/xmlns/foundation/lpa/1.0/Application"; //$NON-NLS-1$
var LPA_DOMAIN_URI = "http://jazz.net/xmlns/foundation/lpa/1.0/domain"; //$NON-NLS-1$
var DC_TITLE = "http://purl.org/dc/terms/title"; //$NON-NLS-1$

function makeRemoteCall(dfd, callback, errback) {
	dfd.addBoth(function(result) {
		if(result instanceof Error) {
			if (errback) {
				errback(result.jfsHttpException.message);
			} else {
				var msgType = result.jfsHttpException.msgType;
				
				dojo.publish(TOPIC_DISPLAY_MSG, [{
					content: result.jfsHttpException.message,
					type: (msgType && msgType === "info") ? MessageBox.STATUS_TYPES.INFO : MessageBox.STATUS_TYPES.ERROR //$NON-NLS-1$
				}]);
			}
		} else {
			callback(result);
		}
	});
}

dojo.declare("com.ibm.team.lpa.app.ui.internal.ContextPickerDialog", null, { //$NON-NLS-1$
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.lpa.app.ui.internal", "ContextPickerDialogMessages"); //$NON-NLS-2$ //$NON-NLS-1$
		
		this.onAdd = args.onAdd;
		
		var dialogTitle = args.dialogTitle || this.messages.defaultTitle;
		
		this.dialog = new jazz.ui.Dialog({
			contentNode: (new com.ibm.team.lpa.app.ui.internal.ContextPickerForm({
				domain: args.domain,
				mode: args.mode,
				defaultApplicationUri: args.defaultApplicationUri,
				curContainers: args.curContainers,
				onOk: dojo.hitch(this, this.add),
				onCancel: dojo.hitch(this, this.cancel)
			})).domNode,
			destroyContents: true,
			primaryTitle: dialogTitle,
			width: "40em" //$NON-NLS-1$
		});
	},
	
	cancel: function() {
		this.dialog.close();
	},
	
	add: function(result) {
		this.onAdd(result);
		this.dialog.close();
	}	
});

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
dojo.require("dojo.io.iframe"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.lpa.app.ui.internal", "ProjectWidgetMessages"); //$NON-NLS-2$ //$NON-NLS-1$

dojo.declare("com.ibm.team.lpa.app.ui.internal.ContextPickerForm", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$

	templatePath: dojo.moduleUrl("com.ibm.team.lpa.app","ui/internal/templates/ContextPickerForm.html"), //$NON-NLS-2$ //$NON-NLS-1$
		
	defaultMode: "ADD_CONTAINER", //$NON-NLS-1$
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.lpa.app.ui.internal", "ContextPickerDialogMessages"); //$NON-NLS-2$ //$NON-NLS-1$
		
		this._mode = this.defaultMode;
		if (args.mode && com.ibm.team.lpa.app.ui.internal.ContextPickerDialog.MODE_TYPES[args.mode]){
			this._mode = args.mode;
		}
		
		this._curContainers = args.curContainers;
		this._domain = args.domain;
		this._applicationMap = {};
		this.defaultApplicationUri = args.defaultApplicationUri;
	},
	
	postCreate: function() {
		if (!this.textDir) {
			this.textDir = jazz.app.i18n.getPreferredBaseTextDir();
		}
		var messages = this.messages;
		if (this._mode === com.ibm.team.lpa.app.ui.internal.ContextPickerDialog.MODE_TYPES.TEMPLATE_CONTAINER){
			var templateInstructions = dojo.string.substitute(messages.instructionsForTemplate, [this._domain.label]);
			dojo.place(document.createTextNode(templateInstructions), this._instructionsDiv);
		}  else if (this._mode === com.ibm.team.lpa.app.ui.internal.ContextPickerDialog.MODE_TYPES.ADD_CONTAINER){
			dojo.place(document.createTextNode(messages.dialogInstructions), this._instructionsDiv);
			this._containerSelect.multiple = "multiple"; //$NON-NLS-1$
		} else if (this._mode === com.ibm.team.lpa.app.ui.internal.ContextPickerDialog.MODE_TYPES.CHANGE_CONTAINER){
			dojo.place(document.createTextNode(messages.dialogInstructionsForChange), this._instructionsDiv);
		}
		
		this.subscribe(TOPIC_DISPLAY_MSG, "displayMessage"); //$NON-NLS-1$
		
		this.refresh();
	},
	
	refresh: function(){
		this._applicationMap = {};
		this._domainObjMap = {};
		
		dojo.empty(this._applicationSelect);
		dojo.create("option", { //$NON-NLS-1$
			innerHTML: this.messages.loadingApplications, 
			className: 'j-label', //$NON-NLS-1$
			disabled: true
		}, this._containerSelect);
		
		this._buildApplicationsMap();
	},
	
	_buildApplicationsMap: function(){
		this._makeRemoteCall(LifecycleProjectAdminClient.getApplications(), function(applications){
			var noAppsRegistered = true;
			
			for (var subjectUri in applications) {
				var subject = applications[subjectUri];
				var type = subject[RDF_SYNTAX_TYPE_URI][0].value;
				
				if (LPA_APPLICATION_URI === type) {//is the current subject an LPA application?
					noAppsRegistered = false;
					if (this._applicationMap[subjectUri]){
						//if we have a duplicate application, append the URI to the display name
						this._applicationMap[subjectUri].displayName += "(" + uri + ")" //$NON-NLS-2$ //$NON-NLS-1$
					} else {
						//add the application to the map
						var title = subject[DC_TITLE][0].value;
						var domain = subject[LPA_DOMAIN_URI][0].value;
						var uri = subjectUri.replace("/rootservices", ""); //$NON-NLS-2$ //$NON-NLS-1$
						
						var app = {
							displayName: title,
							domain: domain,
							label: title,
							uri: uri
						};
						this._applicationMap[subjectUri] = app;
					}
				}
			} // end loop
			
			if (noAppsRegistered){
				this.displayMessage({
					content: this.messages.noApplicationsMsg,
					type: MessageBox.STATUS_TYPES.ERROR,
					closable: false
				});
				
				dojo.empty(this._containerSelect);
				dojo.create("option", { //$NON-NLS-1$
					value: "",  //$NON-NLS-1$
					innerHTML: this.messages.emptyContextResult, 
					className: 'j-label', //$NON-NLS-1$
					disabled: true
				}, this._containerSelect);
			} else {
				this._buildDomainsMap();	
			}
		});
	},
	
	_buildDomainsMap: function(){
		this._makeRemoteCall(LifecycleProjectAdminClient.getDomains(), function(result){
			dojo.forEach(result, dojo.hitch(this, function(domain){
				this._domainObjMap[domain.uri] = domain;
			}));
			
			this._buildApplicationSelect();
		});
	},
	
	_buildApplicationSelect: function(){
		var applicationMap = this._applicationMap;
		
		dojo.empty(this._applicationSelect);
		for (subjectUri in applicationMap){
			var app = applicationMap[subjectUri];
			if(this._domain && app.domain !== this._domain.uri){
				// only show applications that work within the same domain as the one we provided
				continue;
			}
			
			var option = null;
			
			var domain = this._domainObjMap[app.domain];
			var option = dojo.create("option", { //$NON-NLS-1$
				innerHTML: jazz.util.html.escape(domain.label + " (" + app.label + ")"), //$NON-NLS-2$ //$NON-NLS-1$
				value: app.uri
			}, this._applicationSelect);
			
			
			if (option && this.defaultApplicationUri && this.defaultApplicationUri.replace("/rootservices", "") == app.uri) { //$NON-NLS-2$ //$NON-NLS-1$
				option.selected = true;
			}
		}
		
		this._doApplicationSelected({
			applicationUri: this._applicationSelect.value
		});
	},
	
	//******************************
	// Event Callbacks
	//******************************
	
	onAddContext: function(e){
		dojo.stopEvent(e);
		var selectedContexts = [];
		dojo.forEach(this._containerSelect.options, dojo.hitch(this, function(option) {
			if (option.selected === true && option.disabled !== true) {
				selectedContexts.push(this._contextMap[option.value]); //need the fuckin context map
			}
		}));
		this.onOk({
			application: this._applicationMap[this._applicationSelect.value], 
			contexts: selectedContexts
		});
	},
	
	onCancelPressed: function(e) {
		dojo.stopEvent(e);
		this.onCancel();
	},
	
	onContainerSelect: function(e){
		if(this._containerSelect.value === ""){ //$NON-NLS-1$
			this._addButton.disabled = true;
		} else {
			this._addButton.disabled = false;
		}
	},
	
	onApplicationChange: function(e){
		this._doApplicationSelected({
			applicationUri: this._applicationSelect.value
		});
	},
	
	_doApplicationSelected: function(args){
		var applicationUri = args.applicationUri;
		
		dojo.empty(this._containerSelect);
		dojo.create("option", { //$NON-NLS-1$
			innerHTML: this.messages.loadingContainers, 
			className: 'j-label' //$NON-NLS-1$
		}, this._containerSelect);
		
		this._makeRemoteCall(LifecycleProjectAdminClient.getContexts({
			application: {
				uri: applicationUri + "/rootservices" //$NON-NLS-1$
			}
		}), function(result){
			this._contextMap = {};
			
			this._hideMessages();
			dojo.empty(this._containerSelect);
			
			if (result.length <= 0){
				dojo.create("option", { //$NON-NLS-1$
					value: "",  //$NON-NLS-1$
					innerHTML: this.messages.emptyContextResult, 
					className: 'j-label', //$NON-NLS-1$
					disabled: true
				}, this._containerSelect);
			} else {
				dojo.forEach(result, dojo.hitch(this, function(context){
					this._contextMap[context.uri] = context;
					var option = dojo.create("option", { //$NON-NLS-1$
						innerHTML: jazz.util.html.escape(this.enforceTextDirWithUcc({},context.label)) + " (" + this.messages.projectArea + ")", //$NON-NLS-2$ //$NON-NLS-1$
						value: context.uri
					}, this._containerSelect);
					
					if (this._curContainers[context.uri]){
						option.disabled = true;
					}
				}));
			}
		});
	},
	
	//******************************
	// MessageBox Management
	//******************************
	displayMessage: function(args) {
		var closable = args.closable;
		var content = args.content;
		var type = args.type;
		
		var isClosable = (closable) ? closable : true;
		this.messageBox.showMessage({
			closable: isClosable,
			summary: content,
			type: type
		});
	},

	_hideMessages: function() {
		this.messageBox.hideMessage();
	},
	
	//******************************
	// Util Methods
	//******************************
	
	//
	// Make xhr requests
	// error handling is automatically handled
	//
	_makeRemoteCall: function(dfd, callback) {
		makeRemoteCall(dfd, dojo.hitch(this, callback));
	}
});

com.ibm.team.lpa.app.ui.internal.ContextPickerDialog.MODE_TYPES = {
	TEMPLATE_CONTAINER: "TEMPLATE_CONTAINER", //$NON-NLS-1$
	ADD_CONTAINER: "ADD_CONTAINER", //$NON-NLS-1$
	CHANGE_CONTAINER: "CHANGE_CONTAINER" //$NON-NLS-1$
};

})();